"""For ProjectTaskStatusLink set default roles_for_board

Revision ID: deeacd38d373
Revises: 20dfeb36142b
Create Date: 2024-01-16 23:48:25.874470

"""

from alembic import op
import sqlalchemy as sa
import sqlalchemy_utils
from sqlalchemy.orm.session import Session
from sqlalchemy.ext.declarative import declarative_base
from zou.migrations.utils.base import BaseMixin

# revision identifiers, used by Alembic.
revision = "deeacd38d373"
down_revision = "20dfeb36142b"
branch_labels = None
depends_on = None

base = declarative_base()

ROLE_TYPES = [
    ("user", "Artist"),
    ("admin", "Studio Manager"),
    ("supervisor", "Supervisor"),
    ("manager", "Production Manager"),
    ("client", "Client"),
    ("vendor", "Vendor"),
]


class ProjectTaskStatusLink(base, BaseMixin):
    __tablename__ = "project_task_status_link"
    project_id = sa.Column(
        sqlalchemy_utils.types.UUIDType(binary=False),
        primary_key=True,
    )
    task_status_id = sa.Column(
        sqlalchemy_utils.types.UUIDType(binary=False),
        primary_key=True,
    )
    priority = sa.Column(sa.Integer, default=None)
    roles_for_board = sa.Column(
        sa.ARRAY(sqlalchemy_utils.types.choice.ChoiceType(ROLE_TYPES)),
        default=["user", "admin", "supervisor", "manager", "vendor"],
    )

    __table_args__ = (
        sa.UniqueConstraint(
            "project_id", "task_status_id", name="project_taskstatus_uc"
        ),
    )


def upgrade():
    session = Session(bind=op.get_bind())
    session.query(ProjectTaskStatusLink).update(
        {
            ProjectTaskStatusLink.roles_for_board: [
                "user",
                "admin",
                "supervisor",
                "manager",
                "vendor",
            ]
        }
    )
    session.commit()


def downgrade():
    pass



"""Add entity version model

Revision ID: cf3d365de164
Revises: 9010a64e5a2d
Create Date: 2019-12-13 11:49:02.299674

"""

from alembic import op
import sqlalchemy as sa
import sqlalchemy_utils
from sqlalchemy.dialects import postgresql
import sqlalchemy_utils
import uuid

# revision identifiers, used by Alembic.
revision = "cf3d365de164"
down_revision = "9010a64e5a2d"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "entity_version",
        sa.Column(
            "id",
            sqlalchemy_utils.types.uuid.UUIDType(binary=False),
            default=uuid.uuid4,
            nullable=False,
        ),
        sa.Column("created_at", sa.DateTime(), nullable=True),
        sa.Column("updated_at", sa.DateTime(), nullable=True),
        sa.Column("name", sa.String(length=160), nullable=False),
        sa.Column(
            "data", postgresql.JSONB(astext_type=sa.Text()), nullable=True
        ),
        sa.Column(
            "entity_id",
            sqlalchemy_utils.types.uuid.UUIDType(binary=False),
            default=uuid.uuid4,
            nullable=True,
        ),
        sa.ForeignKeyConstraint(
            ["entity_id"],
            ["entity.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(
        op.f("ix_entity_version_entity_id"),
        "entity_version",
        ["entity_id"],
        unique=False,
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(
        op.f("ix_entity_version_entity_id"), table_name="entity_version"
    )
    op.drop_table("entity_version")
    # ### end Alembic commands ###



from zou.app.models.hardware_item import HardwareItem
from zou.app.blueprints.crud.base import BaseModelResource, BaseModelsResource


class HardwareItemsResource(BaseModelsResource):
    def __init__(self):
        BaseModelsResource.__init__(self, HardwareItem)


class HardwareItemResource(BaseModelResource):
    def __init__(self):
        BaseModelResource.__init__(self, HardwareItem)



from flask import Blueprint
from zou.app.utils.api import configure_api_from_blueprint

from zou.app.blueprints.edits.resources import (
    EditResource,
    EditsResource,
    AllEditsResource,
    EditsAndTasksResource,
    EditPreviewsResource,
    EditTaskTypesResource,
    EditTasksResource,
    EditVersionsResource,
    ProjectEditsResource,
    EpisodeEditsResource,
    EpisodeEditTasksResource,
)

routes = [
    ("/data/edits", AllEditsResource),
    ("/data/edits/all", EditsResource),
    ("/data/edits/with-tasks", EditsAndTasksResource),
    ("/data/edits/<edit_id>", EditResource),
    ("/data/edits/<edit_id>/task-types", EditTaskTypesResource),
    ("/data/edits/<edit_id>/tasks", EditTasksResource),
    ("/data/edits/<edit_id>/preview-files", EditPreviewsResource),
    ("/data/edits/<edit_id>/versions", EditVersionsResource),
    ("/data/episodes/<episode_id>/edits", EpisodeEditsResource),
    ("/data/episodes/<episode_id>/edit-tasks", EpisodeEditTasksResource),
    ("/data/projects/<project_id>/edits", ProjectEditsResource),
]


blueprint = Blueprint("edits", "edits")
api = configure_api_from_blueprint(blueprint, routes)



from flask_restful import Resource

from flask_jwt_extended import jwt_required

from zou.app.mixin import ArgsMixin
from zou.app.utils import permissions
from zou.app.services import index_service, projects_service, user_service


class SearchResource(Resource, ArgsMixin):
    @jwt_required()
    def post(self):
        """
        Search for resource
        ---
        tags:
        - Search
        parameters:
          - in: formData
            name: query
            required: True
            type: string
            example: test will search for test
          - in: formData
            name: limit
            required: False
            type: integer
            default: 3
            example: 3
          - in: formData
            name: index_names
            required: False
            type: list of strings
            default: ["assets", "shots", "persons"]
            example: ["assets"]
        responses:
            200:
                description: List of entities that contain the query
        """
        args = self.get_args(
            [
                ("query", "", True),
                ("project_id", None, False),
                ("limit", 3, False, int),
                ("offset", 0, False, int),
                (
                    "index_names",
                    ["assets", "shots", "persons"],
                    False,
                    str,
                    "append",
                ),
            ]
        )
        query = args["query"]
        limit = args["limit"]
        offset = args["offset"]
        project_id = args["project_id"]
        index_names = args["index_names"]
        results = {}
        if len(query) < 3:
            return results

        if permissions.has_admin_permissions():
            projects = projects_service.open_projects()
        else:
            projects = user_service.get_open_projects()
        project_ids = [project["id"] for project in projects]

        if project_id is not None and len(project_id) > 0:
            if project_id in project_ids:
                project_ids = [project_id]
            else:
                project_ids = []

        if "persons" in index_names:
            results["persons"] = index_service.search_persons(
                query, limit=limit, offset=offset
            )
        if "assets" in index_names:
            if (
                len(project_ids) == 0
                and not permissions.has_admin_permissions()
            ):
                results["assets"] = []
            else:
                results["assets"] = index_service.search_assets(
                    query, project_ids, limit=limit, offset=offset
                )
        if "shots" in index_names:
            if (
                len(project_ids) == 0
                and not permissions.has_admin_permissions()
            ):
                results["shots"] = []
            else:
                results["shots"] = index_service.search_shots(
                    query, project_ids, limit=limit, offset=offset
                )

        return results



"""add timesheets locked field to org

Revision ID: 6d7fa5a8e9a5
Revises: 8e4f39e321f4
Create Date: 2021-05-07 13:19:06.751710

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "6d7fa5a8e9a5"
down_revision = "8e4f39e321f4"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "organisation",
        sa.Column("timesheets_locked", sa.Boolean(), nullable=True),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("organisation", "timesheets_locked")
    # ### end Alembic commands ###



import logging

from alembic import context
from sqlalchemy import create_engine, pool
from logging.config import fileConfig
from flask import current_app

from zou.app import db

# Database URL (passed by Alembic)
config = context.config
url = current_app.config.get("SQLALCHEMY_DATABASE_URI")

# Interpret the config file for Python logging.
# This line sets up loggers basically.
fileConfig(config.config_file_name)
logger = logging.getLogger("alembic.env")


def include_object(object, name, type_, reflected, compare_to):
    if type_ == "table":
        return not reflected or name in db.metadata.tables
    return True


def render_item(type_, obj, autogen_context):
    """Apply custom rendering for selected items."""

    import sqlalchemy_utils

    if type_ == "type" and isinstance(
        obj, sqlalchemy_utils.types.uuid.UUIDType
    ):
        autogen_context.imports.add("import sqlalchemy_utils")
        autogen_context.imports.add("import uuid")
        return "sqlalchemy_utils.types.uuid.UUIDType(binary=False), default=uuid.uuid4"

    return False


def process_revision_directives(context, revision, directives):
    if getattr(config.cmd_opts, "autogenerate", False):
        script = directives[0]
        if script.upgrade_ops.is_empty():
            directives[:] = []
            logger.info("No changes in schema detected.")


def run_migrations_online():
    connectable = create_engine(url, poolclass=pool.NullPool)
    with connectable.connect() as connection:
        context.configure(
            connection=connection,
            target_metadata=db.metadata,
            process_revision_directives=process_revision_directives,
            render_item=render_item,
            include_object=include_object,
            **current_app.extensions["migrate"].configure_args,
        )
        with context.begin_transaction():
            context.run_migrations()


run_migrations_online()



import datetime
import urllib.parse

from calendar import monthrange
from dateutil import relativedelta

from sqlalchemy.exc import StatementError

from babel.dates import format_datetime

from flask_jwt_extended import create_access_token, get_jti, current_user

from zou.app.models.department import Department
from zou.app.models.desktop_login_log import DesktopLoginLog
from zou.app.models.organisation import Organisation
from zou.app.models.person import Person
from zou.app.models.time_spent import TimeSpent

from zou.app import config, file_store
from zou.app.utils import fields, events, cache, emails, date_helpers
from zou.app.services import index_service, auth_service
from zou.app.stores import auth_tokens_store
from zou.app.services.exception import (
    PersonNotFoundException,
    PersonInProtectedAccounts,
    WrongParameterException,
)


def clear_person_cache():
    cache.cache.delete_memoized(get_person)
    cache.cache.delete_memoized(get_person_by_email)
    cache.cache.delete_memoized(get_person_by_desktop_login)
    cache.cache.delete_memoized(get_person_by_email_desktop_login)
    cache.cache.delete_memoized(get_active_persons)
    cache.cache.delete_memoized(get_persons)


def clear_organisation_cache():
    cache.cache.delete_memoized(get_organisation)
    cache.cache.delete_memoized(get_organisation, True)


@cache.memoize_function(120)
def get_persons(minimal=False):
    """
    Return all person stored in database.
    """
    persons = []
    for person in Person.query.all():
        if minimal:
            persons.append(person.present_minimal(relations=True))
        else:
            persons.append(person.serialize_safe(relations=True))
    return persons


def get_all_raw_active_persons():
    """
    Return all person stored in database without serialization.
    """
    return Person.get_all_by(active=True)


@cache.memoize_function(120)
def get_active_persons():
    """
    Return all persons with flag active set to True.
    """
    persons = (
        Person.query.filter_by(active=True)
        .order_by(Person.first_name)
        .order_by(Person.last_name)
        .all()
    )
    return fields.serialize_models(persons)


def get_person_raw(person_id):
    """
    Return given person as an active record.
    """
    if person_id is None:
        raise PersonNotFoundException()

    try:
        person = Person.get(person_id)
    except StatementError:
        raise PersonNotFoundException()

    if person is None:
        raise PersonNotFoundException()
    return person


@cache.memoize_function(120)
def get_person(person_id, unsafe=False, relations=True):
    """
    Return given person as a dictionary.
    """
    person = get_person_raw(person_id)
    if unsafe:
        return person.serialize(relations=relations)
    else:
        return person.serialize_safe(relations=relations)


def get_person_by_email_raw(email):
    """
    Return person that matches given email as an active record.
    """
    person = Person.get_by(email=email, is_bot=False)

    if person is None:
        raise PersonNotFoundException()
    return person


@cache.memoize_function(120)
def get_person_by_email(email, unsafe=False, relations=False):
    """
    Return person that matches given email as a dictionary.
    """
    person = get_person_by_email_raw(email)
    if unsafe:
        return person.serialize(relations=relations)
    else:
        return person.serialize_safe(relations=relations)


@cache.memoize_function(120)
def get_person_by_desktop_login(desktop_login):
    """
    Return person that matches given desktop login as a dictionary. It is useful
    to authenticate user from their desktop session login.
    """
    try:
        person = Person.get_by(desktop_login=desktop_login, is_bot=False)
    except StatementError:
        raise PersonNotFoundException()

    if person is None:
        raise PersonNotFoundException()
    return person.serialize()


def get_current_user(unsafe=False, relations=False):
    """
    Return person from its auth token (the one that does the request) as a
    dictionary.
    """
    if unsafe:
        return current_user.serialize(relations=relations)
    else:
        return current_user.serialize_safe(relations=relations)


def get_current_user_raw():
    """
    Return person from its auth token (the one that does the request) as an
    active record.
    """
    return current_user


def get_person_by_ldap_uid(ldap_uid):
    """
    Return person that matches given ldap_uid as a dictionary.
    """
    if ldap_uid is None:
        raise PersonNotFoundException()
    try:
        person = Person.get_by(ldap_uid=ldap_uid, is_bot=False)
    except StatementError:
        raise PersonNotFoundException()

    if person is None:
        raise PersonNotFoundException()
    return person.serialize()


@cache.memoize_function(120)
def get_person_by_email_desktop_login(email_or_desktop_login):
    """
    Return person that matches given email or desktop login as a dictionary.
    """
    try:
        return get_person_by_email(email_or_desktop_login, unsafe=True)
    except PersonNotFoundException:
        return get_person_by_desktop_login(email_or_desktop_login)


def get_persons_map():
    """
    Return a dict of which keys are person_id and values are person.
    """
    persons = Person.query.all()
    return {str(person.id): person.serialize_safe() for person in persons}


def create_person(
    email,
    password,
    first_name,
    last_name,
    phone="",
    role="user",
    desktop_login="",
    departments=[],
    is_generated_from_ldap=False,
    ldap_uid=None,
    is_bot=False,
    expiration_date=None,
    studio_id=None,
    active=True,
    serialize=True,
):
    """
    Create a new person entry in the database. No operation are performed on
    password, so encrypted password is expected.
    """
    if email is not None:
        email = email.strip()

    if expiration_date is not None:
        try:
            if (
                date_helpers.get_date_from_string(expiration_date).date()
                < datetime.date.today()
            ):
                raise WrongParameterException(
                    "Expiration date can't be in the past."
                )
        except WrongParameterException:
            raise
        except:
            raise WrongParameterException("Expiration date is not valid.")

    person = Person.create(
        email=email,
        password=password,
        first_name=first_name,
        last_name=last_name,
        phone=phone,
        role=role,
        desktop_login=desktop_login,
        departments=departments,
        is_generated_from_ldap=is_generated_from_ldap,
        ldap_uid=ldap_uid,
        is_bot=is_bot,
        expiration_date=expiration_date,
        studio_id=studio_id,
        active=active,
    )
    if is_bot:
        access_token = create_access_token_for_raw_person(person)
    index_service.index_person(person)
    events.emit("person:new", {"person_id": person.id})
    clear_person_cache()
    if serialize:
        if is_bot:
            return {
                "access_token": access_token,
                **person.serialize(relations=True),
            }
        else:
            return person.serialize(relations=True)
    else:
        return person


def update_password(email, password):
    """
    Update password field for use matching given email.
    """
    person = get_person_by_email_raw(email)
    person.update({"password": password})
    clear_person_cache()
    return person.serialize()


def update_person(person_id, data, bypass_protected_accounts=False):
    """
    Update person entry with data given in parameter.
    """
    person = Person.get(person_id)
    if (
        not bypass_protected_accounts
        and person.email in config.PROTECTED_ACCOUNTS
    ):
        message = None
        if data.get("active") is False:
            message = (
                "Can't set this person as inactive it's a protected account."
            )
        elif data.get("role") is not None:
            message = "Can't change the role of this person it's a protected account."

        if message is not None:
            raise PersonInProtectedAccounts(message)

    if "email" in data and data["email"] is not None:
        data["email"] = data["email"].strip()

    if "expiration_date" in data and data["expiration_date"] is not None:
        try:
            if (
                datetime.datetime.strptime(
                    data["expiration_date"], "%Y-%m-%d"
                ).date()
                < datetime.date.today()
            ):
                raise WrongParameterException(
                    "Expiration date can't be in the past."
                )
        except WrongParameterException:
            raise
        except:
            raise WrongParameterException("Expiration date is not valid.")

    person.update(data)

    if "expiration_date" in data:
        access_token = create_access_token_for_raw_person(person)
    index_service.remove_person_index(person_id)
    if person.active:
        index_service.index_person(person)
    events.emit("person:update", {"person_id": person_id})
    clear_person_cache()
    if "expiration_date" in data:
        return {
            "access_token": access_token,
            **person.serialize(),
        }
    else:
        return person.serialize()


def delete_person(person_id):
    """
    Delete person entry from database.
    """
    person = Person.get(person_id)
    person_dict = person.serialize()
    person.delete()
    index_service.remove_person_index(person_id)
    events.emit("person:delete", {"person_id": person_id})
    clear_person_cache()
    return person_dict


def get_desktop_login_logs(person_id):
    """
    Get all logs for user desktop logins.
    """
    logs = (
        DesktopLoginLog.query.filter(DesktopLoginLog.person_id == person_id)
        .order_by(DesktopLoginLog.date.desc())
        .all()
    )
    return fields.serialize_list(logs)


def create_desktop_login_logs(person_id, date):
    """
    Add a new log entry for desktop logins.
    """
    log = DesktopLoginLog.create(person_id=person_id, date=date).serialize()
    update_person_last_presence(person_id)
    return log


def update_person_last_presence(person_id):
    """
    Update person presence field with the most recent time spent or
    desktop login log.
    """
    log = DesktopLoginLog.query.order_by(DesktopLoginLog.date.desc()).first()
    time_spent = TimeSpent.query.order_by(TimeSpent.date.desc()).first()
    date = None
    if (
        log is not None
        and time_spent is not None
        and log.date > time_spent.date
    ):
        date = log.date
    elif time_spent is not None:
        date = time_spent.date
    return update_person(
        person_id, {"last_presence": date}, bypass_protected_accounts=True
    )


def get_presence_logs(year, month):
    """
    Return arrays of presence for a given month, adapted for a CSV rendering.
    Rows are users and columns represent the days of given month.
    """
    persons = get_active_persons()
    headers = [str(year)]
    csv_content = []

    (_, limit) = monthrange(year, month)
    headers += [str(i) for i in range(1, limit + 1)]
    start_date = datetime.datetime(year, month, 1, 0, 0, 0)
    end_date = datetime.date.today() + relativedelta.relativedelta(months=1)

    csv_content.append(headers)
    for person in persons:
        row = [person["full_name"]]
        row += ["" for i in range(1, limit + 1)]
        logs = (
            DesktopLoginLog.query.filter(
                DesktopLoginLog.person_id == person["id"]
            )
            .filter(DesktopLoginLog.date >= start_date)
            .filter(DesktopLoginLog.date < end_date)
            .order_by(DesktopLoginLog.date)
            .all()
        )

        for log in logs:
            day = log.date.day
            row[day] = "X"
        csv_content.append(row)
    return csv_content


def is_admin(person):
    return person["role"] == "admin"


def invite_person(person_id):
    """
    Send an invitation email to given person (a mail telling him/her how to
    connect on Kitsu).
    """
    person = get_person(person_id)
    organisation = get_organisation()
    token = auth_service.generate_reset_token()
    auth_tokens_store.add(
        "reset-token-%s" % person["email"], token, ttl=3600 * 24 * 7
    )
    subject = (
        "You are invited by %s to join their Kitsu production tracker"
        % (organisation["name"])
    )
    params = {"email": person["email"], "token": token}
    query = urllib.parse.urlencode(params)
    reset_url = "%s://%s/reset-change-password?%s" % (
        config.DOMAIN_PROTOCOL,
        config.DOMAIN_NAME,
        query,
    )

    time_string = format_datetime(
        date_helpers.get_utc_now_datetime(),
        tzinfo=person["timezone"],
        locale=person["locale"],
    )

    html = f"""<p>Hello {person["first_name"]},</p>
<p>
You are invited by {organisation["name"]} to collaborate on their Kitsu production tracker.
</p>
<p>
Your login is: <strong>{person["email"]}</strong>
</p>
<p>
You are invited to set your password by following this link: <a href="{reset_url}">{reset_url}</a>
</p>
<p>
This link will expire after one week. After, you have to request to reset your password.
The invitation was sent at this date: {time_string}.
</p>
<p>
Thank you and see you soon on Kitsu,
</p>
<p>
{organisation["name"]} Team
</p>
"""

    emails.send_email(subject, html, person["email"])


@cache.memoize_function(120)
def get_organisation(sensitive=False):
    """
    Return organisation set up on this instance. It creates it if none exists.
    """
    organisation = Organisation.query.first()
    if organisation is None:
        organisation = Organisation.create(name="Kitsu")
    return organisation.present(sensitive=sensitive)


def update_organisation(organisation_id, data):
    """
    Update organisation entry with data given in parameter.
    """
    organisation = Organisation.get(organisation_id)
    organisation.update(data)
    events.emit("organisation:update", {"organisation_id": organisation_id})
    clear_organisation_cache()
    return organisation.present()


def is_user_limit_reached():
    """
    Returns true if the number of active users is equal and superior to the
    user limit set in the configuration.
    """
    nb_active_users = Person.query.filter(
        Person.active, Person.is_bot.isnot(True)
    ).count()
    return nb_active_users >= config.USER_LIMIT


def add_to_department(department_id, person_id):
    """
    Add to department.
    """
    person = get_person_raw(person_id)
    department = Department.get(department_id)
    person.departments.append(department)
    person.save()
    clear_person_cache()
    return person.serialize(relations=True)


def remove_from_department(department_id, person_id):
    """
    Remove from department.
    """
    person = get_person_raw(person_id)
    person.departments = [
        department
        for department in person.departments
        if str(department.id) != department_id
    ]
    person.save()
    clear_person_cache()
    return person.serialize(relations=True)


def clear_avatar(person_id):
    """
    Set person `has_avatar` field to False and delete related file.
    """
    person = get_person_raw(person_id)
    person.update({"has_avatar": False})
    clear_person_cache()
    if config.REMOVE_FILES:
        try:
            file_store.remove_picture("thumbnails", person_id)
        except BaseException:
            pass
    return person.serialize()


def is_jti_revoked(jti):
    """
    Return True if the given token id is revoked.
    """
    return Person.query.filter_by(jti=jti).first() is None


def create_access_token_for_raw_person(person):
    """
    Create an access token for the given raw person.
    """
    expires_delta = False
    if person.expiration_date is not None:
        expires_delta = (
            datetime.datetime.combine(
                person.expiration_date,
                datetime.datetime.max.time(),
            )
            - date_helpers.get_utc_now_datetime()
        )
    access_token = create_access_token(
        identity=person.id,
        additional_claims={
            "identity_type": "bot" if person.is_bot else "person_api",
        },
        expires_delta=expires_delta,
    )
    person.jti = get_jti(access_token)
    person.save()
    return access_token



from zou.app.models.serializer import SerializerMixin
from zou.app.models.base import BaseMixin

from sqlalchemy.orm import declarative_base
from sqlalchemy import MetaData, Column, Integer

plugin_metadata = MetaData()
PluginBase = declarative_base(metadata=plugin_metadata)


class Count(PluginBase, BaseMixin, SerializerMixin):
    """
    Describe a plugin.
    """

    __tablename__ = "toto_count"
    count = Column(Integer, nullable=False, default=0)



from flask import Blueprint
from zou.app.utils.api import configure_api_from_blueprint

from zou.app.blueprints.departments.resources import (
    AllDepartmentSoftwareResource,
    AddSoftwareToDepartmentResource,
    SoftwareDepartmentResource,
    AllDepartmentHardwareItemsResource,
    AddHardwareItemToDepartmentResource,
    HardwareItemDepartmentResource,
)

routes = [
    ("/data/departments/software-licenses", AllDepartmentSoftwareResource),
    (
        "/data/departments/<department_id>/software-licenses",
        AddSoftwareToDepartmentResource,
    ),
    (
        "/data/departments/<department_id>/software-licenses/<software_id>",
        SoftwareDepartmentResource,
    ),
    ("/data/departments/hardware-items", AllDepartmentHardwareItemsResource),
    (
        "/data/departments/<department_id>/hardware-items",
        AddHardwareItemToDepartmentResource,
    ),
    (
        "/data/departments/<department_id>/hardware-items/<hardware_item_id>",
        HardwareItemDepartmentResource,
    ),
]

blueprint = Blueprint("departments", "departments")
api = configure_api_from_blueprint(blueprint, routes)



from flask import abort, current_app

from sqlalchemy.exc import StatementError
from flask_jwt_extended import jwt_required

from zou.app.models.output_type import OutputType
from zou.app.blueprints.crud.base import BaseModelResource, BaseModelsResource
from zou.app.services import files_service


class OutputTypesResource(BaseModelsResource):
    def __init__(self):
        BaseModelsResource.__init__(self, OutputType)

    def check_read_permissions(self, options=None):
        return True


class OutputTypeResource(BaseModelResource):
    def __init__(self):
        BaseModelResource.__init__(self, OutputType)

    def check_read_permissions(self, instance):
        return True

    @jwt_required()
    def get(self, instance_id):
        """
        Retrieve an output type corresponding at given ID and return it as a
        JSON object.
        ---
        tags:
          - Crud
        parameters:
          - in: path
            name: output_type_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            200:
                description: Model as a JSON object
            400:
                description: Statement error
            404:
                description: Value error
        """
        try:
            output_type = files_service.get_output_type(instance_id)
            self.check_read_permissions(output_type)
            return self.clean_get_result(output_type)

        except StatementError as exception:
            current_app.logger.error(str(exception), exc_info=1)
            return {"message": str(exception)}, 400

        except ValueError:
            abort(404)

    def post_update(self, instance_dict, data):
        files_service.clear_output_type_cache(instance_dict["id"])
        return instance_dict

    def post_delete(self, instance_dict):
        files_service.clear_output_type_cache(instance_dict["id"])
        return instance_dict



import importlib.util
import logging
import sys
from alembic import context
from sqlalchemy import create_engine, pool
from pathlib import Path
from logging.config import fileConfig

from zou.app.utils.plugins import PluginManifest

plugin_path = Path(__file__).resolve().parents[1]
models_path = plugin_path / "models.py"
manifest = PluginManifest.from_plugin_path(plugin_path)

module_name = f"_plugin_models_{manifest['id']}"
spec = importlib.util.spec_from_file_location(module_name, models_path)
module = importlib.util.module_from_spec(spec)
sys.modules[module_name] = module
spec.loader.exec_module(module)

# Database URL (passed by Alembic)
config = context.config
url = config.get_main_option("sqlalchemy.url")

# Interpret the config file for Python logging.
# This line sets up loggers basically.
fileConfig(config.config_file_name)
logger = logging.getLogger("alembic.env")


def include_object(object, name, type_, reflected, compare_to):
    if type_ == "table":
        return not reflected or name in module.plugin_metadata.tables
    return True


def render_item(type_, obj, autogen_context):
    """Apply custom rendering for selected items."""

    import sqlalchemy_utils

    if type_ == "type" and isinstance(
        obj, sqlalchemy_utils.types.uuid.UUIDType
    ):
        autogen_context.imports.add("import sqlalchemy_utils")
        autogen_context.imports.add("import uuid")
        return "sqlalchemy_utils.types.uuid.UUIDType(binary=False), default=uuid.uuid4"

    return False


def run_migrations_online():
    connectable = create_engine(url, poolclass=pool.NullPool)
    with connectable.connect() as connection:
        context.configure(
            connection=connection,
            target_metadata=module.plugin_metadata,
            version_table=f"alembic_version_{manifest['id']}",
            compare_type=True,
            include_object=include_object,
            render_item=render_item,
        )
        with context.begin_transaction():
            context.run_migrations()


run_migrations_online()



from flask_restful import Resource
from flask import request
from flask_jwt_extended import jwt_required
from slugify import slugify

from zou.app.services import (
    shots_service,
    projects_service,
    tasks_service,
    user_service,
    persons_service,
)
from zou.app.utils import csv_utils, query


class ShotsCsvExport(Resource):
    @jwt_required()
    def get(self, project_id):
        """
        Export shots linked to a given project as csv.
        ---
        tags:
          - Export
        parameters:
          - in: path
            name: project_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            200:
                description: Shots exported as csv
        """
        project = projects_service.get_project(project_id)
        self.check_permissions(project["id"])

        criterions = query.get_query_criterions_from_request(request)
        criterions["project_id"] = project["id"]

        self.task_status_map = tasks_service.get_task_status_map()
        self.task_type_map = tasks_service.get_task_type_map()
        self.persons_map = persons_service.get_persons_map()

        csv_content = []
        results = self.get_shots_data(criterions)
        metadata_infos = self.get_metadata_infos(project_id)
        validation_columns = self.get_validation_columns(results)
        headers = self.build_headers(metadata_infos, validation_columns)
        csv_content.append(headers)

        for result in results:
            result["project_name"] = project["name"]
            csv_content.append(
                self.build_row(result, metadata_infos, validation_columns)
            )

        file_name = "%s shots" % project["name"]
        return csv_utils.build_csv_response(csv_content, slugify(file_name))

    def check_permissions(self, project_id):
        user_service.check_project_access(project_id)
        user_service.block_access_to_vendor()

    def build_headers(self, metadata_infos, validation_columns):
        headers = [
            "Project",
            "Episode",
            "Sequence",
            "Name",
            "Description",
            "Time Spent",
            "Frames",
            "Frame In",
            "Frame Out",
            "FPS",
        ]

        metadata_headers = [name for (name, _, _) in metadata_infos]

        validation_assignations_columns = []
        for validation_column in validation_columns:
            validation_assignations_columns.append(validation_column)
            validation_assignations_columns.append("Assignations")
        return headers + metadata_headers + validation_assignations_columns

    def get_shots_data(self, criterions):
        results = shots_service.get_shots_and_tasks(criterions)
        return sorted(
            results,
            key=lambda shot: (
                shot["episode_name"],
                shot["sequence_name"],
                shot["name"],
            ),
        )

    def build_row(self, result, metadata_infos, validation_columns):
        row = [
            result["project_name"],
            result["episode_name"],
            result["sequence_name"],
            result["name"],
            result["description"],
            self.get_time_spent(result),
            result["nb_frames"],
            result.get("data", {}).get("frame_in", ""),
            result.get("data", {}).get("frame_out", ""),
            result.get("data", {}).get("fps", ""),
        ]
        task_map = {}

        for task in result["tasks"]:
            task_status = self.task_status_map[task["task_status_id"]]
            task_type = self.task_type_map[task["task_type_id"]]
            task_map[task_type["name"]] = {}
            task_map[task_type["name"]]["short_name"] = task_status[
                "short_name"
            ]
            task_map[task_type["name"]]["assignees"] = ",".join(
                [
                    self.persons_map[person_id]["full_name"]
                    for person_id in task["assignees"]
                ]
            )
        result_data = result.get("data", {}) or {}
        for _, field_name, data_type in metadata_infos:
            result_metadata = result_data.get(field_name, "")
            if data_type == "boolean":
                row.append(
                    "true" if result_metadata.lower() == "true" else "false"
                )
            else:
                row.append(result_metadata)

        for column in validation_columns:
            if column in task_map:
                row.append(task_map[column]["short_name"])
                row.append(task_map[column]["assignees"])
            else:
                row.append("")
                row.append("")

        return row

    def get_validation_columns(self, results):
        validation_map = {}

        for result in results:
            for task in result["tasks"]:
                task_type = self.task_type_map[task["task_type_id"]]
                validation_map[task_type["name"]] = {
                    "name": task_type["name"],
                    "priority": task_type["priority"],
                }

        validation_columns = [
            validation["name"]
            for validation in sorted(
                validation_map.values(),
                key=lambda validation: (
                    validation["priority"],
                    validation["name"],
                ),
            )
        ]

        return validation_columns

    def get_metadata_infos(self, project_id):
        descriptors = [
            descriptor
            for descriptor in projects_service.get_metadata_descriptors(
                project_id
            )
            if descriptor["entity_type"] == "Shot"
        ]

        columns = [
            (
                descriptor["name"],
                descriptor["field_name"],
                descriptor["data_type"],
            )
            for descriptor in descriptors
        ]

        return columns

    def get_time_spent(self, result):
        time_spent = 0
        for task in result["tasks"]:
            if task["duration"] is not None:
                time_spent += task["duration"]

        if time_spent > 0:
            time_spent = time_spent / 8.0 / 60.0

        return "%.2f" % time_spent



from zou.app.models.event import ApiEvent
from zou.app.models.login_log import LoginLog
from zou.app.utils import fields
from sqlalchemy import func


def get_last_events(
    after=None,
    before=None,
    limit=100,
    only_files=False,
    project_id=None,
    name=None,
):
    """
    Return last 100 events published. If before parameter is set, it returns
    last 100 events before this date.
    """
    query = ApiEvent.query.order_by(ApiEvent.created_at.desc())

    if after is not None:
        query = query.filter(
            ApiEvent.created_at > func.cast(after, ApiEvent.created_at.type)
        )

    if before is not None:
        query = query.filter(
            ApiEvent.created_at < func.cast(before, ApiEvent.created_at.type)
        )

    if only_files:
        query = query.filter(
            ApiEvent.name.in_(
                (
                    "preview-file:add-file",
                    "organisation:set-thumbnail",
                    "person:set-thumbnail",
                    "project:set-thumbnail",
                )
            )
        )

    if project_id is not None:
        query = query.filter(ApiEvent.project_id == project_id)

    if name is not None:
        query = query.filter(ApiEvent.name == name)

    events = query.limit(limit).all()
    return [
        fields.serialize_dict(
            {
                "id": event.id,
                "created_at": event.created_at,
                "name": event.name,
                "user_id": event.user_id,
                "data": event.data,
            }
        )
        for event in events
    ]


def create_login_log(person_id, ip_address, origin):
    """
    Create a new entry to register that someone logged in.
    """
    login_log = LoginLog.create(
        person_id=person_id, ip_address=ip_address, origin=origin
    )
    return login_log.serialize()


def get_last_login_logs(before=None, limit=100):
    """
    Return last 100 login logs published. If before parameter is set, it returns
    last 100 login logs before this date.
    """
    query = LoginLog.query.order_by(LoginLog.created_at.desc()).with_entities(
        LoginLog.created_at, LoginLog.ip_address, LoginLog.person_id
    )

    if before is not None:
        query = query.filter(
            LoginLog.created_at < func.cast(before, LoginLog.created_at.type)
        )

    login_logs = query.limit(limit).all()
    return [
        {
            "created_at": fields.serialize_value(created_at),
            "ip_address": fields.serialize_value(ip_address),
            "person_id": fields.serialize_value(person_id),
        }
        for (created_at, ip_address, person_id) in login_logs
    ]



from zou.app.models.playlist import Playlist
from zou.app.services import user_service, playlists_service

from zou.app.blueprints.crud.base import BaseModelResource, BaseModelsResource
from zou.app.utils import fields


class PlaylistsResource(BaseModelsResource):
    def __init__(self):
        BaseModelsResource.__init__(self, Playlist)

    def check_read_permissions(self, options=None):
        return True

    def check_create_permissions(self, playlist):
        user_service.check_supervisor_project_access(playlist["project_id"])

    def update_data(self, data):
        data = super().update_data(data)
        if "episode_id" in data and data["episode_id"] in ["all", "main"]:
            data["episode_id"] = None
        if "task_type_id" in data and not fields.is_valid_id(
            data["task_type_id"]
        ):
            data["task_type_id"] = None
        return data


class PlaylistResource(BaseModelResource):
    def __init__(self):
        BaseModelResource.__init__(self, Playlist)

    def check_read_permissions(self, playlist):
        user_service.check_project_access(playlist["project_id"])
        user_service.block_access_to_vendor()

    def check_update_permissions(self, playlist, data):
        user_service.check_project_access(playlist["project_id"])
        user_service.block_access_to_vendor()

    def pre_update(self, instance_dict, data):
        if "shots" in data:
            shots = [
                {
                    "entity_id": shot.get("entity_id", shot.get("id", "")),
                    "preview_file_id": shot["preview_file_id"],
                }
                for shot in data["shots"]
                if "preview_file_id" in shot
            ]
            data["shots"] = shots
        return data

    def delete(self, instance_id):
        playlists_service.remove_playlist(instance_id)
        return "", 204



"""
This module is named source instead of import because import is a Python
keyword.
"""

from flask import Blueprint
from zou.app.utils.api import configure_api_from_blueprint

from zou.app.blueprints.index.resources import (
    ConfigResource,
    IndexResource,
    InfluxStatusResource,
    StatusResource,
    StatusResourcesResource,
    StatsResource,
    TestEventsResource,
    TxtStatusResource,
)

routes = [
    ("/", IndexResource),
    ("/status", StatusResource),
    ("/status/influx", InfluxStatusResource),
    ("/status/resources", StatusResourcesResource),
    ("/status.txt", TxtStatusResource),
    ("/status/test-event", TestEventsResource),
    ("/stats", StatsResource),
    ("/config", ConfigResource),
]

blueprint = Blueprint("index", "index")
api = configure_api_from_blueprint(blueprint, routes)



from flask_restful import Resource
from .models import Count


class HelloWorld(Resource):
    def get(self):
        if not Count.query.first():
            c = Count.create()
        else:
            c = Count.query.first()
            c.count += 1
            Count.commit()
        return {"message": "Hello, World!", "count": c.count}



from flask import Blueprint
from zou.app.utils.api import configure_api_from_blueprint

from zou.app.blueprints.persons.resources import (
    DateTimeSpentsResource,
    DayOffResource,
    DayOffForMonthResource,
    DesktopLoginsResource,
    InvitePersonResource,
    PersonMonthQuotaShotsResource,
    PersonWeekQuotaShotsResource,
    PersonDayQuotaShotsResource,
    PersonMonthTimeSpentsResource,
    PersonMonthAllTimeSpentsResource,
    PersonWeekTimeSpentsResource,
    PersonDayTimeSpentsResource,
    PersonWeekDayOffResource,
    PersonYearDayOffResource,
    PersonDayOffResource,
    PersonMonthDayOffResource,
    PresenceLogsResource,
    TimeSpentsResource,
    TimeSpentMonthResource,
    TimeSpentMonthsResource,
    TimeSpentWeekResource,
    TimeSpentYearsResource,
    PersonYearTimeSpentsResource,
    AddToDepartmentResource,
    RemoveFromDepartmentResource,
    ChangePasswordForPersonResource,
    DisableTwoFactorAuthenticationPersonResource,
    ClearAvatarPersonResource,
)

routes = [
    ("/data/persons/<person_id>/desktop-login-logs", DesktopLoginsResource),
    ("/data/persons/presence-logs/<month_date>", PresenceLogsResource),
    ("/data/persons/<person_id>/time-spents", TimeSpentsResource),
    ("/data/persons/<person_id>/time-spents/<date>", DateTimeSpentsResource),
    ("/data/persons/<person_id>/day-offs/<date>", DayOffResource),
    (
        "/data/persons/<person_id>/time-spents/year/<year>",
        PersonYearTimeSpentsResource,
    ),
    (
        "/data/persons/<person_id>/time-spents/month/<year>/<month>",
        PersonMonthTimeSpentsResource,
    ),
    (
        "/data/persons/<person_id>/time-spents/month/all/<year>/<month>",
        PersonMonthAllTimeSpentsResource,
    ),
    (
        "/data/persons/<person_id>/time-spents/week/<year>/<week>",
        PersonWeekTimeSpentsResource,
    ),
    (
        "/data/persons/<person_id>/time-spents/day/<year>/<month>/<day>",
        PersonDayTimeSpentsResource,
    ),
    (
        "/data/persons/<person_id>/quota-shots/month/<year>/<month>",
        PersonMonthQuotaShotsResource,
    ),
    (
        "/data/persons/<person_id>/quota-shots/week/<year>/<week>",
        PersonWeekQuotaShotsResource,
    ),
    (
        "/data/persons/<person_id>/quota-shots/day/<year>/<month>/<day>",
        PersonDayQuotaShotsResource,
    ),
    ("/data/persons/time-spents/year-table/", TimeSpentYearsResource),
    ("/data/persons/time-spents/month-table/<year>", TimeSpentMonthsResource),
    ("/data/persons/time-spents/week-table/<year>", TimeSpentWeekResource),
    (
        "/data/persons/time-spents/day-table/<year>/<month>",
        TimeSpentMonthResource,
    ),
    ("/data/persons/day-offs/<year>/<month>", DayOffForMonthResource),
    (
        "/data/persons/<person_id>/day-offs/week/<year>/<week>",
        PersonWeekDayOffResource,
    ),
    (
        "/data/persons/<person_id>/day-offs/month/<year>/<month>",
        PersonMonthDayOffResource,
    ),
    (
        "/data/persons/<person_id>/day-offs/year/<year>",
        PersonYearDayOffResource,
    ),
    (
        "/data/persons/<person_id>/day-offs",
        PersonDayOffResource,
    ),
    ("/actions/persons/<person_id>/invite", InvitePersonResource),
    ("/actions/persons/<person_id>/departments/add", AddToDepartmentResource),
    (
        "/actions/persons/<person_id>/departments/<department_id>",
        RemoveFromDepartmentResource,
    ),
    (
        "/actions/persons/<person_id>/change-password",
        ChangePasswordForPersonResource,
    ),
    (
        "/actions/persons/<person_id>/disable-two-factor-authentication",
        DisableTwoFactorAuthenticationPersonResource,
    ),
    (
        "/actions/persons/<person_id>/clear-avatar",
        ClearAvatarPersonResource,
    ),
]

blueprint = Blueprint("persons", "persons")
api = configure_api_from_blueprint(blueprint, routes)



"""add department keys to filter models

Revision ID: 9d3bb33c6fc6
Revises: 8e67c183bed7
Create Date: 2024-10-11 15:52:29.740222

"""

from alembic import op
import sqlalchemy as sa
import sqlalchemy_utils
import sqlalchemy_utils
import uuid

# revision identifiers, used by Alembic.
revision = "9d3bb33c6fc6"
down_revision = "8e67c183bed7"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("search_filter", schema=None) as batch_op:
        batch_op.add_column(
            sa.Column(
                "department_id",
                sqlalchemy_utils.types.uuid.UUIDType(binary=False),
                default=uuid.uuid4,
                nullable=True,
            )
        )
        batch_op.create_foreign_key(
            "search_filter_department_id_fkey",
            "department",
            ["department_id"],
            ["id"],
        )

    with op.batch_alter_table("search_filter_group", schema=None) as batch_op:
        batch_op.add_column(
            sa.Column(
                "department_id",
                sqlalchemy_utils.types.uuid.UUIDType(binary=False),
                default=uuid.uuid4,
                nullable=True,
            )
        )
        batch_op.create_foreign_key(
            "search_filter_group_department_id_fkey",
            "department",
            ["department_id"],
            ["id"],
        )

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("search_filter_group", schema=None) as batch_op:
        batch_op.drop_constraint(
            "search_filter_group_department_id_fkey", type_="foreignkey"
        )
        batch_op.drop_column("department_id")

    with op.batch_alter_table("search_filter", schema=None) as batch_op:
        batch_op.drop_constraint(
            "search_filter_department_id_fkey", type_="foreignkey"
        )
        batch_op.drop_column("department_id")

    # ### end Alembic commands ###



from zou.app.models.department import (
    Department,
    SoftwareDepartmentLink,
    HardwareItemDepartmentLink,
)
from zou.app.models.hardware_item import HardwareItem
from zou.app.models.software import Software
from zou.app.utils import fields

from zou.app.services.exception import (
    DepartmentNotFoundException,
    SoftwareNotFoundException,
    HardwareItemNotFoundException,
)


def _check_department_exists(department_id):
    department = Department.get(department_id)
    if not department:
        raise DepartmentNotFoundException
    return department


def _check_software_exists(software_id):
    software = Software.get(software_id)
    if not software:
        raise SoftwareNotFoundException
    return software


def _check_hardware_item_exists(hardware_item_id):
    hardware_item = HardwareItem.get(hardware_item_id)
    if not hardware_item:
        raise HardwareItemNotFoundException
    return hardware_item


def get_all_software_for_departments():
    """
    Get all software items for all departments organized by department
    in a dictionary where the key is the department id and the value is a
    list of linked software items.
    """
    software_list = (
        Software.query.join(SoftwareDepartmentLink)
        .add_columns(SoftwareDepartmentLink.department_id)
        .all()
    )
    department_map = {}
    for software, department_id in software_list:
        if department_id not in department_map:
            department_map[department_id] = []
        department_map[department_id].append(software.serialize())
    return department_map


def get_all_hardware_items_for_departments():
    """
    Get all hardware items for all departments organized by department
    in a dictionary where the key is the department id and the value is a
    list of linked hardware items.
    """
    hardware_item_list = (
        HardwareItem.query.join(HardwareItemDepartmentLink)
        .add_columns(HardwareItemDepartmentLink.department_id)
        .all()
    )
    department_map = {}
    for hardware_item, department_id in hardware_item_list:
        if department_id not in department_map:
            department_map[department_id] = []
        department_map[department_id].append(hardware_item.serialize())
    return department_map


def get_software_for_department(department_id):
    """
    Get all software items for a given department.
    """
    _check_department_exists(department_id)
    return fields.serialize_models(
        Software.query.join(SoftwareDepartmentLink)
        .filter(SoftwareDepartmentLink.department_id == department_id)
        .all()
    )


def get_hardware_items_for_department(department_id):
    """
    Get all hardware items for a given department.
    """
    _check_department_exists(department_id)
    return fields.serialize_models(
        HardwareItem.query.join(HardwareItemDepartmentLink)
        .filter(HardwareItemDepartmentLink.department_id == department_id)
        .all()
    )


def add_software_to_department(department_id, software_id):
    """
    Add a software item to a department.
    """
    _check_department_exists(department_id)
    _check_software_exists(software_id)
    link = SoftwareDepartmentLink.get_or_create(
        department_id=department_id, software_id=software_id
    )
    return link.serialize()


def remove_software_from_department(department_id, software_id):
    """
    Remove a software item from a department.
    """
    _check_department_exists(department_id)
    _check_software_exists(software_id)
    link = SoftwareDepartmentLink.get_by(
        department_id=department_id, software_id=software_id
    )
    if not link:
        return None
    else:
        link.delete()
        return link.serialize()


def add_hardware_item_to_department(department_id, hardware_item_id):
    """
    Add a hardware item to a department.
    """
    _check_department_exists(department_id)
    _check_hardware_item_exists(hardware_item_id)
    link = HardwareItemDepartmentLink.get_or_create(
        department_id=department_id, hardware_item_id=hardware_item_id
    )
    return link.serialize()


def remove_hardware_item_from_department(department_id, hardware_item_id):
    """
    Remove a hardware item from a department.
    """
    _check_department_exists(department_id)
    _check_hardware_item_exists(hardware_item_id)
    link = HardwareItemDepartmentLink.get_by(
        department_id=department_id, hardware_item_id=hardware_item_id
    )
    if not link:
        return None
    else:
        link.delete()
        return link.serialize()



"""Add for client flag to playlist

Revision ID: 98c90621cf58
Revises: 5b7fa3e51701
Create Date: 2019-11-04 20:10:43.256743

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "98c90621cf58"
down_revision = "5b7fa3e51701"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "playlist",
        sa.Column("for_client", sa.Boolean(), nullable=True, default=False),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("playlist", "for_client")
    # ### end Alembic commands ###



from sqlalchemy_utils import UUIDType, ChoiceType

from sqlalchemy.dialects.postgresql import JSONB

from zou.app import db
from zou.app.models.serializer import SerializerMixin
from zou.app.models.base import BaseMixin

from zou.app.models.person import ROLE_TYPES

PROJECT_STYLES = [
    ("2d", "2D Animation"),
    ("2dpaper", "2D Animation (Paper)"),
    ("3d", "3D Animation"),
    ("2d3d", "2D/3D Animation"),
    ("ar", "Augmented Reality"),
    ("vfx", "VFX"),
    ("stop-motion", "Stop Motion"),
    ("motion-design", "Motion Design"),
    ("archviz", "Archviz"),
    ("commercial", "Commercial"),
    ("catalog", "Catalog"),
    ("immersive", "Immersive Experience"),
    ("nft", "NFT Collection"),
    ("video-game", "Video Game"),
    ("vr", "Virtual Reality"),
]


class ProjectPersonLink(db.Model):
    __tablename__ = "project_person_link"
    project_id = db.Column(
        UUIDType(binary=False),
        db.ForeignKey("project.id"),
        primary_key=True,
        index=True,
    )
    person_id = db.Column(
        UUIDType(binary=False),
        db.ForeignKey("person.id"),
        primary_key=True,
        index=True,
    )
    shotgun_id = db.Column(db.Integer)


class ProjectTaskTypeLink(db.Model, BaseMixin, SerializerMixin):
    __tablename__ = "project_task_type_link"
    project_id = db.Column(
        UUIDType(binary=False),
        db.ForeignKey("project.id"),
        primary_key=True,
        index=True,
    )
    task_type_id = db.Column(
        UUIDType(binary=False),
        db.ForeignKey("task_type.id"),
        primary_key=True,
        index=True,
    )
    priority = db.Column(db.Integer, default=None)

    __table_args__ = (
        db.UniqueConstraint(
            "project_id", "task_type_id", name="project_tasktype_uc"
        ),
    )


class ProjectTaskStatusLink(db.Model, BaseMixin, SerializerMixin):
    __tablename__ = "project_task_status_link"
    project_id = db.Column(
        UUIDType(binary=False),
        db.ForeignKey("project.id"),
        primary_key=True,
        index=True,
    )
    task_status_id = db.Column(
        UUIDType(binary=False),
        db.ForeignKey("task_status.id"),
        primary_key=True,
        index=True,
    )
    priority = db.Column(db.Integer, default=None)
    roles_for_board = db.Column(
        db.ARRAY(ChoiceType(ROLE_TYPES)),
        default=["user", "admin", "supervisor", "manager", "vendor"],
    )

    __table_args__ = (
        db.UniqueConstraint(
            "project_id", "task_status_id", name="project_taskstatus_uc"
        ),
    )


class ProjectAssetTypeLink(db.Model):
    __tablename__ = "project_asset_type_link"
    project_id = db.Column(
        UUIDType(binary=False), db.ForeignKey("project.id"), primary_key=True
    )
    asset_type_id = db.Column(
        UUIDType(binary=False),
        db.ForeignKey("entity_type.id"),
        primary_key=True,
    )


class ProjectStatusAutomationLink(db.Model):
    __tablename__ = "project_status_automation_link"
    project_id = db.Column(
        UUIDType(binary=False),
        db.ForeignKey("project.id"),
        primary_key=True,
        index=True,
    )
    status_automation_id = db.Column(
        UUIDType(binary=False),
        db.ForeignKey("status_automation.id"),
        primary_key=True,
        index=True,
    )


class ProjectPreviewBackgroundFileLink(db.Model):
    __tablename__ = "project_preview_background_file_link"
    project_id = db.Column(
        UUIDType(binary=False), db.ForeignKey("project.id"), primary_key=True
    )
    preview_background_file_id = db.Column(
        UUIDType(binary=False),
        db.ForeignKey("preview_background_file.id"),
        primary_key=True,
    )


class Project(db.Model, BaseMixin, SerializerMixin):
    """
    Describes a production the studio works on.
    """

    name = db.Column(db.String(80), nullable=False, unique=True, index=True)
    code = db.Column(db.String(80))
    description = db.Column(db.Text())
    shotgun_id = db.Column(db.Integer)
    file_tree = db.Column(JSONB)
    data = db.Column(JSONB)
    has_avatar = db.Column(db.Boolean(), default=False)
    fps = db.Column(db.String(10), default=25)
    ratio = db.Column(db.String(10), default="16:9")
    resolution = db.Column(db.String(12), default="1920x1080")
    production_type = db.Column(db.String(20), default="short")
    production_style = db.Column(
        ChoiceType(PROJECT_STYLES), default="2d3d", nullable=False
    )
    start_date = db.Column(db.Date())
    end_date = db.Column(db.Date())
    man_days = db.Column(db.Integer)
    nb_episodes = db.Column(db.Integer, default=0)
    episode_span = db.Column(db.Integer, default=0)
    max_retakes = db.Column(db.Integer, default=0)
    is_clients_isolated = db.Column(db.Boolean(), default=False)
    is_preview_download_allowed = db.Column(db.Boolean(), default=False)
    is_set_preview_automated = db.Column(db.Boolean(), default=False)
    homepage = db.Column(db.String(80), default="assets")
    is_publish_default_for_artists = db.Column(db.Boolean(), default=False)
    hd_bitrate_compression = db.Column(db.Integer, default=28)
    ld_bitrate_compression = db.Column(db.Integer, default=6)

    project_status_id = db.Column(
        UUIDType(binary=False), db.ForeignKey("project_status.id"), index=True
    )

    default_preview_background_file_id = db.Column(
        UUIDType(binary=False),
        db.ForeignKey("preview_background_file.id"),
        default=None,
        index=True,
    )

    from_schedule_version_id = db.Column(
        UUIDType(binary=False),
        db.ForeignKey(
            "production_schedule_version.id",
            name="project_from_schedule_version_id_fkey",
            use_alter=True,
        ),
        index=True,
        nullable=True,
    )

    team = db.relationship("Person", secondary=ProjectPersonLink.__table__)
    asset_types = db.relationship(
        "EntityType", secondary=ProjectAssetTypeLink.__table__
    )
    task_statuses = db.relationship(
        "TaskStatus", secondary=ProjectTaskStatusLink.__table__
    )
    task_types = db.relationship(
        "TaskType", secondary=ProjectTaskTypeLink.__table__
    )
    status_automations = db.relationship(
        "StatusAutomation", secondary=ProjectStatusAutomationLink.__table__
    )
    preview_background_files = db.relationship(
        "PreviewBackgroundFile",
        secondary=ProjectPreviewBackgroundFileLink.__table__,
    )

    def set_team(self, person_ids):
        for person_id in person_ids:
            link = ProjectPersonLink.query.filter_by(
                project_id=self.id, person_id=person_id
            ).first()
            if link is None:
                link = ProjectPersonLink(
                    project_id=self.id, person_id=person_id
                )
                db.session.add(link)
        db.session.commit()

    def set_task_types(self, task_type_ids):
        return self.set_links(
            task_type_ids, ProjectTaskTypeLink, "project_id", "task_type_id"
        )

    def set_task_statuses(self, task_status_ids):
        return self.set_links(
            task_status_ids,
            ProjectTaskStatusLink,
            "project_id",
            "task_status_id",
        )

    def set_asset_types(self, asset_type_ids):
        return self.set_links(
            asset_type_ids, ProjectAssetTypeLink, "project_id", "asset_type_id"
        )

    def set_status_automations(self, status_automation_ids):
        return self.set_links(
            status_automation_ids,
            ProjectStatusAutomationLink,
            "project_id",
            "status_automation_id",
        )

    def set_preview_background_files(self, preview_background_files_ids):
        return self.set_links(
            preview_background_files_ids,
            ProjectPreviewBackgroundFileLink,
            "project_id",
            "preview_background_file_id",
        )

    @classmethod
    def create_from_import(cls, data):
        is_update = False
        previous_project = cls.get(data["id"])
        data.pop("team", None)
        data.pop("type", None)
        data.pop("project_status_name", None)
        person_ids = data.pop("team", None)
        task_type_ids = data.pop("task_types", None)
        task_status_ids = data.pop("task_statuses", None)
        asset_type_ids = data.pop("asset_types", None)
        status_automation_ids = data.pop("status_automations", None)
        preview_background_files_ids = data.pop(
            "preview_background_files", None
        )

        if previous_project is None:
            previous_project = cls.create(**data)
            previous_project.save()
        else:
            is_update = True
            previous_project.update(data)
            previous_project.save()

        if person_ids is not None:
            previous_project.set_team(person_ids)

        if task_type_ids is not None:
            previous_project.set_task_types(task_type_ids)

        if task_status_ids is not None:
            previous_project.set_task_statuses(task_status_ids)

        if asset_type_ids is not None:
            previous_project.set_asset_types(asset_type_ids)

        if status_automation_ids is not None:
            previous_project.set_status_automations(status_automation_ids)

        if preview_background_files_ids is not None:
            previous_project.set_preview_background_files(
                preview_background_files_ids
            )

        return (previous_project, is_update)



from zou.app.models.notification import Notification

from zou.app.blueprints.crud.base import BaseModelResource, BaseModelsResource


class NotificationsResource(BaseModelsResource):
    def __init__(self):
        BaseModelsResource.__init__(self, Notification)


class NotificationResource(BaseModelResource):
    def __init__(self):
        BaseModelResource.__init__(self, Notification)



"""Add ProductionScheduleVersionTaskLinkPersonLink

Revision ID: d97f2730bf7b
Revises: 5f715f2b6348
Create Date: 2025-07-16 02:07:13.855652

"""

from alembic import op
import sqlalchemy as sa
import sqlalchemy_utils
import sqlalchemy_utils
import uuid

# revision identifiers, used by Alembic.
revision = "d97f2730bf7b"
down_revision = "5f715f2b6348"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.rename_table("assignations", "task_person_link")
    with op.batch_alter_table(
        "production_schedule_version_task_link", schema=None
    ) as batch_op:
        batch_op.alter_column(
            "production_schedule_version_id",
            existing_type=sa.UUID(),
            nullable=True,
        )
        batch_op.alter_column(
            "task_id", existing_type=sa.UUID(), nullable=True
        )
        batch_op.create_index(
            batch_op.f(
                "ix_production_schedule_version_task_link_production_schedule_version_id"
            ),
            ["production_schedule_version_id"],
            unique=False,
        )
        batch_op.create_index(
            batch_op.f("ix_production_schedule_version_task_link_task_id"),
            ["task_id"],
            unique=False,
        )
    op.create_table(
        "production_schedule_version_task_link_person_link",
        sa.Column(
            "production_schedule_version_task_link_id",
            sqlalchemy_utils.types.uuid.UUIDType(binary=False),
            default=uuid.uuid4,
            nullable=False,
        ),
        sa.Column(
            "person_id",
            sqlalchemy_utils.types.uuid.UUIDType(binary=False),
            default=uuid.uuid4,
            nullable=False,
        ),
        sa.ForeignKeyConstraint(
            ["person_id"],
            ["person.id"],
        ),
        sa.ForeignKeyConstraint(
            ["production_schedule_version_task_link_id"],
            ["production_schedule_version_task_link.id"],
        ),
        sa.PrimaryKeyConstraint(
            "production_schedule_version_task_link_id", "person_id"
        ),
    )

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("production_schedule_version_task_link_person_link")
    with op.batch_alter_table(
        "production_schedule_version_task_link", schema=None
    ) as batch_op:
        batch_op.drop_index(
            batch_op.f("ix_production_schedule_version_task_link_task_id")
        )
        batch_op.drop_index(
            batch_op.f(
                "ix_production_schedule_version_task_link_production_schedule_version_id"
            )
        )
        batch_op.alter_column(
            "task_id", existing_type=sa.UUID(), nullable=False
        )
        batch_op.alter_column(
            "production_schedule_version_id",
            existing_type=sa.UUID(),
            nullable=False,
        )
    op.rename_table("task_person_link", "assignations")
    # ### end Alembic commands ###



"""Add ProductionScheduleVersionTaskLink uc

Revision ID: 0bd1e89f2a6f
Revises: e4b48ca33539
Create Date: 2025-07-22 17:13:16.680191

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "0bd1e89f2a6f"
down_revision = "e4b48ca33539"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table(
        "production_schedule_version_task_link", schema=None
    ) as batch_op:
        batch_op.alter_column(
            "production_schedule_version_id",
            existing_type=sa.UUID(),
            nullable=False,
        )
        batch_op.alter_column(
            "task_id", existing_type=sa.UUID(), nullable=False
        )
        batch_op.create_unique_constraint(
            "production_schedule_version_task_link_uc",
            ["production_schedule_version_id", "task_id"],
        )

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table(
        "production_schedule_version_task_link", schema=None
    ) as batch_op:
        batch_op.drop_constraint(
            "production_schedule_version_task_link_uc", type_="unique"
        )
        batch_op.alter_column(
            "task_id", existing_type=sa.UUID(), nullable=False
        )
        batch_op.alter_column(
            "production_schedule_version_id",
            existing_type=sa.UUID(),
            nullable=False,
        )

    # ### end Alembic commands ###



"""empty message

Revision ID: c726b98be194
Revises: fee7c696166e
Create Date: 2018-08-02 23:20:49.472287

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "c726b98be194"
down_revision = "fee7c696166e"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "preview_file",
        sa.Column("extension", sa.String(length=6), nullable=True),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("preview_file", "extension")
    # ### end Alembic commands ###



"""add project bugdet table

Revision ID: 83e2f33a9b14
Revises: 45f739ef962a
Create Date: 2025-04-09 20:17:08.331320

"""

from alembic import op
import sqlalchemy as sa
import sqlalchemy_utils
import sqlalchemy_utils
import uuid

# revision identifiers, used by Alembic.
revision = "83e2f33a9b14"
down_revision = "45f739ef962a"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "budget",
        sa.Column(
            "project_id",
            sqlalchemy_utils.types.uuid.UUIDType(binary=False),
            default=uuid.uuid4,
            nullable=False,
        ),
        sa.Column("revision", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(length=255), nullable=False),
        sa.Column(
            "id",
            sqlalchemy_utils.types.uuid.UUIDType(binary=False),
            default=uuid.uuid4,
            nullable=False,
        ),
        sa.Column("created_at", sa.DateTime(), nullable=True),
        sa.Column("updated_at", sa.DateTime(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
    )
    with op.batch_alter_table("budget", schema=None) as batch_op:
        batch_op.create_index(
            batch_op.f("ix_budget_project_id"), ["project_id"], unique=False
        )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("budget", schema=None) as batch_op:
        batch_op.drop_index(batch_op.f("ix_budget_project_id"))

    op.drop_table("budget")
    # ### end Alembic commands ###



"""add attachments to message chats


Revision ID: 1fab8c420678
Revises: 92b40d79ad3f
Create Date: 2024-03-28 13:22:39.592100

"""

from alembic import op
import sqlalchemy as sa
import sqlalchemy_utils
import sqlalchemy_utils
import uuid

# revision identifiers, used by Alembic.
revision = "1fab8c420678"
down_revision = "92b40d79ad3f"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("attachment_file", schema=None) as batch_op:
        batch_op.add_column(
            sa.Column(
                "chat_message_id",
                sqlalchemy_utils.types.uuid.UUIDType(binary=False),
                default=uuid.uuid4,
                nullable=True,
            )
        )
        batch_op.drop_constraint("attachment_uc", type_="unique")
        batch_op.create_index(
            batch_op.f("ix_attachment_file_chat_message_id"),
            ["chat_message_id"],
            unique=False,
        )
        batch_op.create_foreign_key(
            None, "chat_message", ["chat_message_id"], ["id"]
        )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("attachment_file", schema=None) as batch_op:
        batch_op.drop_constraint(None, type_="foreignkey")
        batch_op.drop_index(batch_op.f("ix_attachment_file_chat_message_id"))
        batch_op.create_unique_constraint(
            "attachment_uc", ["name", "comment_id"]
        )
        batch_op.drop_column("chat_message_id")

    # ### end Alembic commands ###



"""add archived fields

Revision ID: 8588f254d6b8
Revises: f0c6cbb61869
Create Date: 2022-09-06 14:03:39.729550

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "8588f254d6b8"
down_revision = "f0c6cbb61869"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("person", sa.Column("archived", sa.Boolean(), default=False))
    op.add_column(
        "task_status", sa.Column("archived", sa.Boolean(), default=False)
    )
    op.add_column(
        "task_type", sa.Column("archived", sa.Boolean(), default=False)
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("task_type", "archived")
    op.drop_column("task_status", "archived")
    op.drop_column("person", "archived")
    # ### end Alembic commands ###



"""add people salary information

Revision ID: 2762a797f1f9
Revises: 307edd8c639d
Create Date: 2025-04-01 18:02:39.343857

"""

from alembic import op
import sqlalchemy as sa
import sqlalchemy_utils


# revision identifiers, used by Alembic.
revision = "2762a797f1f9"
down_revision = "307edd8c639d"
branch_labels = None
depends_on = None

from zou.app.models.person import POSITION_TYPES, SENIORITY_TYPES


def upgrade():
    with op.batch_alter_table("person", schema=None) as batch_op:
        batch_op.add_column(
            sa.Column(
                "position",
                sqlalchemy_utils.types.choice.ChoiceType(POSITION_TYPES),
                nullable=True,
            )
        )
        batch_op.add_column(
            sa.Column(
                "seniority",
                sqlalchemy_utils.types.choice.ChoiceType(SENIORITY_TYPES),
                nullable=True,
            )
        )
        batch_op.add_column(
            sa.Column("daily_salary", sa.Integer(), nullable=True)
        )

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("person", schema=None) as batch_op:
        batch_op.drop_column("daily_salary")
        batch_op.drop_column("seniority")
        batch_op.drop_column("position")
    # ### end Alembic commands ###



from flask import current_app

from zou.app.models.project import Project
from zou.app.models.project import ProjectPersonLink
from zou.app.models.person import Person

from zou.app.blueprints.source.shotgun.base import (
    BaseImportShotgunResource,
    ImportRemoveShotgunBaseResource,
)


class ImportShotgunProjectConnectionsResource(BaseImportShotgunResource):
    def __init__(self):
        BaseImportShotgunResource.__init__(self)

    def prepare_import(self):
        pass

    def extract_data(self, sg_project_user_connection):
        sg_project = sg_project_user_connection["project"]
        sg_user = sg_project_user_connection["user"]

        data = {
            "shotgun_id": sg_project_user_connection["id"],
            "project_shotgun_id": sg_project["id"],
            "person_shotgun_id": sg_user["id"],
        }
        return data

    def import_entry(self, data):
        project_person_link = ProjectPersonLink.query.filter(
            ProjectPersonLink.shotgun_id == data["shotgun_id"]
        ).first()

        if project_person_link is None:
            project = Project.get_by(shotgun_id=data["project_shotgun_id"])
            person = Person.get_by(shotgun_id=data["person_shotgun_id"])

            if project is not None and person is not None:
                project.team.append(person)
                project.save()
                current_app.logger.info(
                    "Project Person Link created: %s" % project
                )
        else:
            project.update(data)
            current_app.logger.info("Project updated: %s" % project)

        return project


class ImportRemoveShotgunProjectConnectionResource(
    ImportRemoveShotgunBaseResource
):
    def __init__(self):
        ImportRemoveShotgunBaseResource.__init__(self, ProjectPersonLink)



"""Add Project.production_schedule_version_id

Revision ID: e4b48ca33539
Revises: 26f96f65cfa3
Create Date: 2025-07-22 02:57:31.987773

"""

from alembic import op
import sqlalchemy as sa
import sqlalchemy_utils
import uuid

# revision identifiers, used by Alembic.
revision = "e4b48ca33539"
down_revision = "26f96f65cfa3"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("project", schema=None) as batch_op:
        batch_op.add_column(
            sa.Column(
                "from_schedule_version_id",
                sqlalchemy_utils.types.uuid.UUIDType(binary=False),
                default=uuid.uuid4,
                nullable=True,
            )
        )
        batch_op.create_index(
            batch_op.f("ix_project_from_schedule_version_id"),
            ["from_schedule_version_id"],
            unique=False,
        )
        batch_op.create_foreign_key(
            None,
            "production_schedule_version",
            ["from_schedule_version_id"],
            ["id"],
        )

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("project", schema=None) as batch_op:
        batch_op.drop_constraint(
            "project_from_schedule_version_id_fkey", type_="foreignkey"
        )
        batch_op.drop_index(batch_op.f("ix_project_from_schedule_version_id"))
        batch_op.drop_column("from_schedule_version_id")

    # ### end Alembic commands ###



"""add fields to the software table

Revision ID: 4bd9bfb73f11
Revises: 4d585b6956f2
Create Date: 2025-07-14 12:07:08.700862

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "4bd9bfb73f11"
down_revision = "4d585b6956f2"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("salary_scale", schema=None) as batch_op:
        batch_op.alter_column(
            "salary", existing_type=sa.INTEGER(), nullable=False
        )

    with op.batch_alter_table("software", schema=None) as batch_op:
        batch_op.add_column(
            sa.Column("monthly_cost", sa.Integer(), nullable=True)
        )
        batch_op.add_column(
            sa.Column("inventory_amount", sa.Integer(), nullable=True)
        )

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("software", schema=None) as batch_op:
        batch_op.drop_column("inventory_amount")
        batch_op.drop_column("monthly_cost")

    with op.batch_alter_table("salary_scale", schema=None) as batch_op:
        batch_op.alter_column(
            "salary", existing_type=sa.INTEGER(), nullable=True
        )

    # ### end Alembic commands ###



"""Add milestone model

Revision ID: 6d1b2c60f58b
Revises: 10cf267d95c9
Create Date: 2019-08-22 23:34:32.450079

"""

from alembic import op
import sqlalchemy as sa
import sqlalchemy_utils
import sqlalchemy_utils
import uuid

# revision identifiers, used by Alembic.
revision = "6d1b2c60f58b"
down_revision = "10cf267d95c9"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "milestone",
        sa.Column(
            "id",
            sqlalchemy_utils.types.uuid.UUIDType(binary=False),
            default=uuid.uuid4,
            nullable=False,
        ),
        sa.Column("created_at", sa.DateTime(), nullable=True),
        sa.Column("updated_at", sa.DateTime(), nullable=True),
        sa.Column("date", sa.Date(), nullable=True),
        sa.Column("name", sa.String(length=40), nullable=False),
        sa.Column(
            "project_id",
            sqlalchemy_utils.types.uuid.UUIDType(binary=False),
            default=uuid.uuid4,
            nullable=True,
        ),
        sa.Column(
            "task_type_id",
            sqlalchemy_utils.types.uuid.UUIDType(binary=False),
            default=uuid.uuid4,
            nullable=True,
        ),
        sa.ForeignKeyConstraint(
            ["project_id"],
            ["project.id"],
        ),
        sa.ForeignKeyConstraint(
            ["task_type_id"],
            ["task_type.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(
        op.f("ix_milestone_project_id"),
        "milestone",
        ["project_id"],
        unique=False,
    )
    op.create_index(
        op.f("ix_milestone_task_type_id"),
        "milestone",
        ["task_type_id"],
        unique=False,
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(op.f("ix_milestone_task_type_id"), table_name="milestone")
    op.drop_index(op.f("ix_milestone_project_id"), table_name="milestone")
    op.drop_table("milestone")
    # ### end Alembic commands ###



import slugify

from zou.app.models.preview_background_file import PreviewBackgroundFile
from zou.app.models.entity import Entity
from zou.app.models.entity_type import EntityType
from zou.app.models.metadata_descriptor import MetadataDescriptor
from zou.app.models.person import Person, DepartmentLink
from zou.app.models.project import (
    Project,
    ProjectPersonLink,
    ProjectTaskTypeLink,
    ProjectTaskStatusLink,
)
from zou.app.models.project_status import ProjectStatus
from zou.app.models.status_automation import StatusAutomation
from zou.app.models.task_type import TaskType
from zou.app.models.task_status import TaskStatus
from zou.app.models.department import Department
from zou.app.services import (
    assets_service,
    base_service,
    edits_service,
    shots_service,
)
from zou.app.services.exception import (
    ProjectNotFoundException,
    MetadataDescriptorNotFoundException,
    DepartmentNotFoundException,
    WrongParameterException,
)

from zou.app.utils import fields, events, cache

from sqlalchemy.exc import StatementError
from sqlalchemy.orm.exc import ObjectDeletedError
from sqlalchemy import or_


def clear_project_cache(project_id):
    cache.cache.delete_memoized(get_project, project_id)
    cache.cache.delete_memoized(get_project, project_id, True)
    cache.cache.delete_memoized(get_project_by_name)
    cache.cache.delete_memoized(open_projects)


@cache.memoize_function(120)
def open_projects(name=None):
    """
    Return all open projects. Allow to filter projects by name.
    """
    query = (
        Project.query.join(
            ProjectStatus, Project.project_status_id == ProjectStatus.id
        )
        .outerjoin(MetadataDescriptor)
        .filter(ProjectStatus.name.in_(("Active", "open", "Open")))
        .order_by(Project.name)
    )

    if name is not None:
        query = query.filter(Project.name == name)

    return get_projects_with_extra_data(query)


def open_project_ids():
    """
    Return all open project ids. Allow to filter projects by name.
    """
    return [project["id"] for project in open_projects()]


def get_projects_with_extra_data(
    query, for_client=False, vendor_departments=None
):
    """
    Helpers function to attach:
    * First episode name to current project when it's a TV Show.
    * Add metadata descriptors for this project.
    """
    projects = []
    for project in query.all():
        project_dict = project.serialize(relations=True)
        if for_client:
            descriptors = MetadataDescriptor.get_all_by(
                project_id=project.id, for_client=True
            )
        elif vendor_departments is not None:
            descriptors = MetadataDescriptor.query.filter(
                or_(
                    MetadataDescriptor.departments == None,
                    MetadataDescriptor.departments.any(
                        Department.id.in_(vendor_departments)
                    ),
                )
            )
        else:
            descriptors = MetadataDescriptor.get_all_by(project_id=project.id)
        project_dict["descriptors"] = []
        for descriptor in descriptors:
            project_dict["descriptors"].append(
                {
                    "id": fields.serialize_value(descriptor.id),
                    "name": descriptor.name,
                    "field_name": descriptor.field_name,
                    "data_type": fields.serialize_value(descriptor.data_type),
                    "choices": descriptor.choices,
                    "for_client": descriptor.for_client or False,
                    "entity_type": descriptor.entity_type,
                    "departments": [
                        str(department.id)
                        for department in descriptor.departments
                    ],
                }
            )

        project_dict["task_types_priority"] = {
            str(task_type_link.task_type_id): task_type_link.priority
            for task_type_link in ProjectTaskTypeLink.query.filter_by(
                project_id=project.id
            )
        }

        project_dict["task_statuses_link"] = {
            str(task_status_link.task_status_id): {
                "priority": task_status_link.priority,
                "roles_for_board": fields.serialize_list(
                    task_status_link.roles_for_board
                ),
            }
            for task_status_link in ProjectTaskStatusLink.query.filter_by(
                project_id=project.id
            )
        }

        if project.production_type == "tvshow":
            first_episode = (
                Entity.query.join(EntityType)
                .filter(EntityType.name == "Episode")
                .filter(Entity.project_id == project.id)
                .filter(Entity.status == "running")
                .order_by(Entity.name)
                .first()
            )
            if first_episode is None:
                first_episode = (
                    Entity.query.join(EntityType)
                    .filter(EntityType.name == "Episode")
                    .filter(Entity.project_id == project.id)
                    .order_by(Entity.name)
                    .first()
                )

            if first_episode is not None:
                project_dict["first_episode_id"] = fields.serialize_value(
                    first_episode.id
                )

        projects.append(project_dict)
    return projects


def get_projects():
    """
    Return all projects. Allow to filter projects by name.
    """
    query = (
        Project.query.join(
            ProjectStatus, Project.project_status_id == ProjectStatus.id
        )
        .add_columns(ProjectStatus.name)
        .order_by(Project.name)
    )

    result = []
    for entry in query.all():
        (project, project_status_name) = entry
        data = project.serialize()
        data["project_status_name"] = project_status_name
        result.append(data)

    return result


@cache.memoize_function(480)
def get_project_statuses():
    return fields.serialize_models(ProjectStatus.get_all())


def get_or_create_open_status():
    """
    Return open status. If it does not exist, it creates it.
    """
    return get_or_create_status("Open")


@cache.memoize_function(480)
def get_open_status():
    """
    Return open status. If it does not exist, it creates it.
    """
    return get_or_create_status("Open")


@cache.memoize_function(120)
def get_closed_status():
    """
    Return closed status. If it does not exist, it creates it.
    """
    return get_or_create_status("Closed")


def get_or_create_status(name):
    """
    Return given status. If it does not exist, it creates it.
    """
    project_status = ProjectStatus.get_by(name=name)
    if project_status is None:
        project_status = ProjectStatus(name=name, color="#000000")
        project_status.save()
    return project_status.serialize()


def save_project_status(project_statuses):
    """
    Save in database all project status given in parameter.
    """
    result = []
    filtered_satuses = (x for x in project_statuses if x is not None)

    for status in filtered_satuses:
        project_status = get_or_create_status(status)
        result.append(project_status)
    return result


def get_or_create_project(name):
    """
    Get project which match given name. Create it if it does not exist.
    """
    project = Project.get_by(name=name)
    if project is None:
        open_status = get_or_create_open_status()
        project = Project(name=name, project_status_id=open_status["id"])
        project.save()
    return project.serialize()


def get_project_raw(project_id):
    """
    Get project matching given id, as active record. Raises an exception if
    project is not found.
    """
    try:
        project = Project.get(project_id)
    except StatementError:
        raise ProjectNotFoundException()

    if project is None:
        raise ProjectNotFoundException()

    return project


@cache.memoize_function(240)
def get_project(project_id, relations=False):
    """
    Get project matching given id, as a dict. Raises an exception if project is
    not found.
    """
    return get_project_raw(project_id).serialize(relations=relations)


@cache.memoize_function(120)
def get_project_by_name(project_name):
    """
    Get project matching given name. Raises an exception if project is not
    found.
    """
    project = Project.query.filter(Project.name.ilike(project_name)).first()

    if project is None:
        raise ProjectNotFoundException()

    return project.serialize()


def update_project(project_id, data):
    """
    Update project matching given id with data from *data* dict.
    """
    project = get_project_raw(project_id)
    project.update(data)
    clear_project_cache(project_id)
    events.emit("project:update", {}, project_id=project_id)
    return project.serialize()


def add_team_member(project_id, person_id):
    """
    Add a person listed in database to the the project team.
    """
    return _add_to_list_attr(project_id, Person, person_id, "team")


def remove_team_member(project_id, person_id):
    """
    Remove a person listed in database from the the project team.
    """
    return _remove_from_list_attr(project_id, Person, person_id, "team")


def add_asset_type_setting(project_id, asset_type_id):
    """
    Add an asset type listed in database to the the project asset types.
    """
    return _add_to_list_attr(
        project_id, EntityType, asset_type_id, "asset_types"
    )


def remove_asset_type_setting(project_id, asset_type_id):
    """
    Remove an asset type listed in database from the the project asset types.
    """
    return _remove_from_list_attr(
        project_id, EntityType, asset_type_id, "asset_types"
    )


def add_task_type_setting(project_id, task_type_id, priority=None):
    """
    Add a task type listed in database to the the project task types.
    """
    link = ProjectTaskTypeLink.get_by(
        task_type_id=task_type_id, project_id=project_id
    )
    if not link:
        ProjectTaskTypeLink.create(
            task_type_id=task_type_id, project_id=project_id, priority=priority
        )
    return _save_project(get_project_raw(project_id))


def remove_task_type_setting(project_id, task_type_id):
    """
    Remove a task status listed in database from the the project task types.
    """
    return _remove_from_list_attr(
        project_id, TaskType, task_type_id, "task_types"
    )


def add_task_status_setting(project_id, task_status_id):
    """
    Add a task status listed in database to the the project task statuses.
    """
    return _add_to_list_attr(
        project_id, TaskStatus, task_status_id, "task_statuses"
    )


def remove_task_status_setting(project_id, task_status_id):
    """
    Remove a task status listed in database from the the project task statuses.
    """
    return _remove_from_list_attr(
        project_id, TaskStatus, task_status_id, "task_statuses"
    )


def add_status_automation_setting(project_id, status_automation_id):
    """
    Add a status automation listed in database to the project status automations.
    """
    return _add_to_list_attr(
        project_id,
        StatusAutomation,
        status_automation_id,
        "status_automations",
    )


def remove_status_automation_setting(project_id, status_automation_id):
    """
    Remove a status automation listed in database from the project status
    automations.
    """
    return _remove_from_list_attr(
        project_id,
        StatusAutomation,
        status_automation_id,
        "status_automations",
    )


def add_preview_background_file_setting(
    project_id, preview_background_file_id
):
    """
    Add a preview background file listed in database to the project backgrounds.
    """
    return _add_to_list_attr(
        project_id,
        PreviewBackgroundFile,
        preview_background_file_id,
        "preview_background_files",
    )


def remove_preview_background_file_setting(
    project_id, preview_background_file_id
):
    """
    Remove a preview background file listed in database from the project backgrounds.
    """
    return _remove_from_list_attr(
        project_id,
        PreviewBackgroundFile,
        preview_background_file_id,
        "preview_background_files",
    )


def _add_to_list_attr(project_id, model_class, model_id, list_attr):
    project = get_project_raw(project_id)
    model = model_class.get(model_id)
    if str(model.id) not in [str(m.id) for m in getattr(project, list_attr)]:
        getattr(project, list_attr).append(model)
        return _save_project(project)
    else:
        return project.serialize()


def _remove_from_list_attr(project_id, model_class, model_id, list_attr):
    project = get_project_raw(project_id)
    model = model_class.get(model_id)
    try:
        getattr(project, list_attr).remove(model)
    except ValueError:
        pass
    return _save_project(project)


def _save_project(project):
    project.save()
    clear_project_cache(str(project.id))
    events.emit("project:update", {}, project_id=str(project.id))
    return project.serialize()


def add_metadata_descriptor(
    project_id,
    entity_type,
    name,
    data_type,
    choices,
    for_client,
    departments=[],
):
    if not departments:
        departments = []

    try:
        departments_objects = [
            Department.get(department_id)
            for department_id in departments
            if department_id is not None
        ]
    except StatementError:
        raise DepartmentNotFoundException()

    try:
        descriptor = MetadataDescriptor.create(
            project_id=project_id,
            entity_type=entity_type,
            name=name,
            data_type=data_type,
            choices=choices,
            for_client=for_client,
            departments=departments_objects,
            field_name=slugify.slugify(name, separator="_"),
        )
    except Exception:
        raise WrongParameterException("Metadata descriptor already exists.")
    events.emit(
        "metadata-descriptor:new",
        {"metadata_descriptor_id": str(descriptor.id)},
        project_id=project_id,
    )
    clear_project_cache(project_id)
    return descriptor.serialize()


def get_metadata_descriptors(project_id, for_client=False):
    """
    Get all metadata descriptors for given project and entity type.
    """
    query = MetadataDescriptor.query.filter(
        MetadataDescriptor.project_id == project_id
    ).order_by(MetadataDescriptor.name)
    if for_client:
        query = query.filter(MetadataDescriptor.for_client == True)

    descriptors = query.all()
    return fields.serialize_models(descriptors, relations=True)


def get_metadata_descriptor_raw(metadata_descriptor_id):
    """
    Get metadata descriptor for given id as active record.
    """
    return base_service.get_instance(
        MetadataDescriptor,
        metadata_descriptor_id,
        MetadataDescriptorNotFoundException,
    )


def get_metadata_descriptor(metadata_descriptor_id):
    """
    Get metadata descriptor for given id as dict.
    """
    return get_metadata_descriptor_raw(metadata_descriptor_id).serialize(
        relations=True
    )


def update_metadata_descriptor(metadata_descriptor_id, changes):
    """
    Update metadata descriptor information for given id.
    """
    descriptor = get_metadata_descriptor_raw(metadata_descriptor_id)

    if "name" in changes and len(changes["name"]) > 0:
        changes["field_name"] = slugify.slugify(changes["name"], separator="_")
        if descriptor.field_name != changes["field_name"]:
            query = Entity.query.filter(
                Entity.project_id == descriptor.project_id
            )
            if descriptor.entity_type == "Shot":
                shot_type = shots_service.get_shot_type()
                query = query.filter(Entity.entity_type_id == shot_type["id"])
            elif descriptor.entity_type == "Asset":
                query = query.filter(assets_service.build_asset_type_filter())
            elif descriptor.entity_type == "Edit":
                edit_type = edits_service.get_edit_type()
                query = query.filter(Entity.entity_type_id == edit_type["id"])

            entities = query.all()
            for entity in entities:
                metadata = fields.serialize_value(entity.data) or {}
                value = metadata.pop(descriptor.field_name, None)
                if value is not None:
                    metadata[changes["field_name"]] = value
                    entity.update_no_commit({"data": metadata})
            Entity.commit()

    if "departments" in changes:
        if not changes["departments"]:
            changes["departments"] = []

        try:
            departments_objects = [
                Department.get(department_id)
                for department_id in changes["departments"]
                if department_id is not None
            ]
        except StatementError:
            raise DepartmentNotFoundException()

        changes["departments"] = departments_objects

    descriptor.update(changes)
    events.emit(
        "metadata-descriptor:update",
        {"metadata_descriptor_id": str(descriptor.id)},
        project_id=descriptor.project_id,
    )
    clear_project_cache(str(descriptor.project_id))
    return descriptor.serialize()


def remove_metadata_descriptor(metadata_descriptor_id):
    """
    Delete metadata descriptor and related informations.
    """
    descriptor = get_metadata_descriptor_raw(metadata_descriptor_id)
    entities = Entity.get_all_by(project_id=descriptor.project_id)
    for entity in entities:
        metadata = fields.serialize_value(entity.data)
        if metadata is not None:
            metadata.pop(descriptor.field_name, None)
            entity.update({"data": metadata})
    try:
        descriptor.delete()
    except ObjectDeletedError:
        pass
    events.emit(
        "metadata-descriptor:delete",
        {"metadata_descriptor_id": str(descriptor.id)},
        project_id=descriptor.project_id,
    )
    clear_project_cache(str(descriptor.project_id))
    return descriptor.serialize()


def is_tv_show(project):
    return project["production_type"] == "tvshow"


def is_open(project):
    open_status = get_open_status()
    return project["project_status_id"] == open_status["id"]


def create_project_task_type_link(project_id, task_type_id, priority):
    task_type_link = ProjectTaskTypeLink.get_by(
        project_id=project_id, task_type_id=task_type_id
    )

    if priority is not None:
        priority = int(priority)

    if task_type_link is None:
        task_type_link = ProjectTaskTypeLink.create(
            project_id=project_id, task_type_id=task_type_id, priority=priority
        )
    else:
        task_type_link.update({"priority": priority})

    return task_type_link.serialize()


def create_project_task_status_link(
    project_id, task_status_id, priority, roles_for_board=[]
):
    task_status_link = ProjectTaskStatusLink.get_by(
        project_id=project_id, task_status_id=task_status_id
    )

    if priority is not None:
        priority = int(priority)

    if task_status_link is None:
        task_status_link = ProjectTaskStatusLink.create(
            project_id=project_id,
            task_status_id=task_status_id,
            priority=priority,
            roles_for_board=roles_for_board,
        )
    else:
        task_status_link.update(
            {"priority": priority, "roles_for_board": roles_for_board}
        )

    return task_status_link.serialize()


def get_project_task_types(project_id):
    project = get_project_raw(project_id)
    return Project.serialize_list(project.task_types)


def get_project_task_statuses(project_id):
    project = get_project_raw(project_id)
    return Project.serialize_list(project.task_statuses)


def get_project_status_automations(project_id):
    project = get_project_raw(project_id)
    return Project.serialize_list(project.status_automations)


def get_project_preview_background_files(project_id):
    project = get_project_raw(project_id)
    return Project.serialize_list(project.preview_background_files)


def get_project_fps(project_id):
    """
    Return fps set at project level or default fps if it not set.
    """
    project = get_project(project_id)
    return float(project["fps"] or "25.00")


def get_task_type_priority_map(project_id, for_entity="Asset"):
    """
    Return a dict allowing to match a task type id with a priority.
    """
    task_types = (
        ProjectTaskTypeLink.query.join(TaskType)
        .filter(ProjectTaskTypeLink.project_id == project_id)
        .filter(TaskType.for_entity == for_entity)
    ).all()
    return {
        str(task_type_link.task_type_id): task_type_link.priority
        for task_type_link in task_types
    }


def get_task_type_links(project_id, for_entity="Asset"):
    """
    Return a lisk of links for given project and entity type.
    """
    task_type_links = (
        ProjectTaskTypeLink.query.join(TaskType)
        .filter(ProjectTaskTypeLink.project_id == project_id)
        .filter(TaskType.for_entity == for_entity)
        .order_by(ProjectTaskTypeLink.priority.desc())
    ).all()
    return ProjectTaskTypeLink.serialize_list(task_type_links)


def get_department_team(project_id, department_id):
    persons = (
        Person.query.join(
            ProjectPersonLink, ProjectPersonLink.person_id == Person.id
        )
        .join(DepartmentLink, DepartmentLink.person_id == Person.id)
        .filter(ProjectPersonLink.project_id == project_id)
        .filter(DepartmentLink.department_id == department_id)
    ).all()
    return persons



"""add currency field to budgets

Revision ID: 7a16258f2fab
Revises: 8ab98c178903
Create Date: 2025-04-21 22:27:22.703525

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "7a16258f2fab"
down_revision = "8ab98c178903"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("budget", schema=None) as batch_op:
        batch_op.add_column(sa.Column("currency", sa.String(length=3)))
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("budget", schema=None) as batch_op:
        batch_op.drop_column("currency")
    # ### end Alembic commands ###



"""empty message

Revision ID: 389cfb9de776
Revises: a65bdadbae2f
Create Date: 2019-02-05 18:43:19.270283

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "389cfb9de776"
down_revision = "a65bdadbae2f"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "task", sa.Column("last_comment_date", sa.DateTime(), nullable=True)
    )
    op.add_column(
        "task", sa.Column("retake_count", sa.Integer(), nullable=True)
    )
    op.add_column(
        "task_status", sa.Column("is_retake", sa.Boolean(), nullable=True)
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("task_status", "is_retake")
    op.drop_column("task", "retake_count")
    op.drop_column("task", "last_comment_date")
    # ### end Alembic commands ###



"""Add StatusAutomation.import_last_revision

Revision ID: 57222395f2be
Revises: 5b980f0dc365
Create Date: 2024-05-07 01:23:21.597644

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "57222395f2be"
down_revision = "5b980f0dc365"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("status_automation", schema=None) as batch_op:
        batch_op.add_column(
            sa.Column(
                "import_last_revision",
                sa.Boolean(),
                nullable=True,
                default=False,
                server_default=sa.sql.expression.false(),
            )
        )

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("status_automation", schema=None) as batch_op:
        batch_op.drop_column("import_last_revision")

    # ### end Alembic commands ###



from flask_restful import Resource
from flask_jwt_extended import jwt_required

from zou.app.utils import permissions
from zou.app.mixin import ArgsMixin
from zou.app.services import (
    departments_service,
)


class AllDepartmentSoftwareResource(Resource, ArgsMixin):

    @jwt_required()
    @permissions.require_admin
    def get(self):
        """
        Get all software items for all departments.
        ---
        tags:
        - Departments
        responses:
            200:
                description: A dictionary of departments with their software items
        """
        softwares = departments_service.get_all_software_for_departments()
        return softwares, 200


class AddSoftwareToDepartmentResource(Resource, ArgsMixin):

    @jwt_required()
    @permissions.require_admin
    def post(self, department_id):
        """
        Add a software item to given department.
        ---
        tags:
        - Departments
        parameters:
          - in: path
            name: department_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: body
            name: software_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            201:
                description: User added to given department
            404:
                description: Department ID or Software ID matches no department or software
        """
        args = self.get_args(
            [
                ("software_id", None, True),
            ]
        )
        self.check_id_parameter(department_id)
        self.check_id_parameter(args["software_id"])
        software = departments_service.add_software_to_department(
            department_id, args["software_id"]
        )
        return software, 201


class SoftwareDepartmentResource(Resource, ArgsMixin):

    @jwt_required()
    @permissions.require_admin
    def get(self, department_id):
        """
        Get all software items for a given department.
        ---
        tags:
        - Departments
        parameters:
          - in: path
            name: department_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            200:
                description: Software items for given department
            404:
                description: Department ID matches no department
        """
        self.check_id_parameter(department_id)
        softwares = departments_service.get_softwares_for_department(
            department_id
        )
        return softwares, 200

    @jwt_required()
    @permissions.require_admin
    def delete(self, department_id, software_id):
        """
        Remove a software item from given department.
        ---
        tags:
        - Departments
        parameters:
          - in: path
            name: software_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: path
            name: department_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            204:
                description: User removed from given department
            404:
                description: Department or software ID matches no department or software
        """
        self.check_id_parameter(department_id)
        self.check_id_parameter(software_id)
        departments_service.remove_software_from_department(
            department_id, software_id
        )
        return "", 204


class AllDepartmentHardwareItemsResource(Resource, ArgsMixin):

    @jwt_required()
    @permissions.require_admin
    def get(self):
        """
        Get all hardware items for all departments.
        ---
        tags:
        - Departments
        responses:
            200:
                description: A dictionary of departments with their hardware items
        """
        hardware_items = (
            departments_service.get_all_hardware_items_for_departments()
        )
        return hardware_items, 200


class AddHardwareItemToDepartmentResource(Resource, ArgsMixin):

    @jwt_required()
    @permissions.require_admin
    def post(self, department_id):
        """
        Add a hardware item to given department.
        ---
        tags:
        - Departments
        parameters:
          - in: path
            name: department_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: body
            name: hardware_item_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            201:
                description: Hardware item added to given department
            404:
                description: Department ID matches no department
            400:
                description: Hardware item ID matches no hardware item
        """
        args = self.get_args(
            [
                ("hardware_item_id", None, True),
            ]
        )
        self.check_id_parameter(department_id)
        self.check_id_parameter(args["hardware_item_id"])
        hardware_item_link = (
            departments_service.add_hardware_item_to_department(
                department_id, args["hardware_item_id"]
            )
        )
        return hardware_item_link, 201


class HardwareItemDepartmentResource(Resource, ArgsMixin):

    @jwt_required()
    @permissions.require_admin
    def get(self, department_id):
        """
        Get all hardware items for a given department.
        ---
        tags:
        - Departments
        parameters:
          - in: path
            name: department_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            200:
                description: Hardware items for given department
            404:
                description: Department ID matches no department
        """
        self.check_id_parameter(department_id)
        hardware_items = departments_service.get_hardware_items_for_department(
            department_id
        )
        return hardware_items, 200

    @jwt_required()
    @permissions.require_admin
    def delete(self, department_id, hardware_item_id):
        """
        Remove a hardware item from given department.
        ---
        tags:
        - Departments
        parameters:
          - in: path
            name: department_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: path
            name: hardware_item_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            204:
                description: User removed from given department
            404:
                description: Department or software ID matches no department or software
        """
        self.check_id_parameter(department_id)
        self.check_id_parameter(hardware_item_id)
        departments_service.remove_hardware_item_from_department(
            department_id, hardware_item_id
        )
        return "", 204



"""Remove unique constraint for TaskStatus.is_default

Revision ID: 05ac7e8caa21
Revises: 0cf5e0e035fa
Create Date: 2022-06-16 14:26:54.695564

"""

from alembic import op
from sqlalchemy import orm
import sqlalchemy as sa
from sqlalchemy.ext.declarative import declarative_base
from zou.migrations.utils.base import BaseMixin

# revision identifiers, used by Alembic.
revision = "05ac7e8caa21"
down_revision = "0cf5e0e035fa"
branch_labels = None
depends_on = None


class TaskStatus(declarative_base(), BaseMixin):
    """
    Describe the state of a task. A status marked as reviewable expects a
    preview file linked to relate comment.
    """

    __tablename__ = "task_status"
    name = sa.Column(sa.String(40), nullable=False)
    short_name = sa.Column(
        sa.String(10), unique=True, nullable=False, index=True
    )
    color = sa.Column(sa.String(7), nullable=False)

    is_done = sa.Column(sa.Boolean(), default=False, index=True)
    is_artist_allowed = sa.Column(sa.Boolean(), default=True)
    is_client_allowed = sa.Column(sa.Boolean(), default=True)
    is_retake = sa.Column(sa.Boolean(), default=False)
    is_feedback_request = sa.Column(sa.Boolean(), default=False, index=True)
    is_default = sa.Column(sa.Boolean(), default=False, index=True)
    shotgun_id = sa.Column(sa.Integer)


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index("ix_task_status_is_default", table_name="task_status")
    op.create_index(
        op.f("ix_task_status_is_default"),
        "task_status",
        ["is_default"],
        unique=False,
    )
    # ### end Alembic commands ###

    # set all the TaskStatus.is_default == None to False
    bind = op.get_bind()
    session = orm.Session(bind=bind)
    task_statuses = (
        session.query(TaskStatus).filter(TaskStatus.is_default == None).all()
    )
    for task_status in task_statuses:
        task_status.is_default = False

    session.commit()


def downgrade():
    bind = op.get_bind()
    session = orm.Session(bind=bind)
    task_statuses = (
        session.query(TaskStatus).filter(TaskStatus.is_default == False).all()
    )
    for task_status in task_statuses:
        task_status.is_default = None

    session.commit()
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(op.f("ix_task_status_is_default"), table_name="task_status")
    op.create_index(
        "ix_task_status_is_default",
        "task_status",
        ["is_default"],
        unique=False,
    )
    # ### end Alembic commands ###



from flask import abort, current_app

from sqlalchemy.exc import StatementError
from flask_jwt_extended import jwt_required

from zou.app.utils import events
from zou.app.models.output_file import OutputFile
from zou.app.models.entity import Entity
from zou.app.models.project import Project
from zou.app.services import user_service, entities_service, files_service
from zou.app.utils import permissions

from zou.app.blueprints.crud.base import BaseModelsResource, BaseModelResource


class OutputFilesResource(BaseModelsResource):
    def __init__(self):
        BaseModelsResource.__init__(self, OutputFile)

    def check_read_permissions(self, options=None):
        user_service.block_access_to_vendor()
        return True

    def add_project_permission_filter(self, query):
        if permissions.has_admin_permissions():
            return query
        else:
            query = (
                query.join(Entity, OutputFile.entity_id == Entity.id)
                .join(Project)
                .filter(user_service.build_related_projects_filter())
            )
            return query

    def emit_create_event(self, instance_dict):
        entity = entities_service.get_entity(instance_dict["entity_id"])
        return events.emit(
            "output-file:new",
            {"output_file_id": instance_dict["id"]},
            project_id=entity["project_id"],
        )


class OutputFileResource(BaseModelResource):
    def __init__(self):
        BaseModelResource.__init__(self, OutputFile)

    def check_read_permissions(self, instance):
        entity = entities_service.get_entity(instance["entity_id"])
        user_service.check_project_access(entity["project_id"])
        user_service.check_entity_access(entity["id"])
        return True

    def check_update_permissions(self, output_file, data):
        if permissions.has_manager_permissions():
            return True
        else:
            return user_service.check_working_on_entity(
                output_file["temporal_entity_id"] or output_file["entity_id"]
            )

    @jwt_required()
    def get(self, instance_id):
        """
        Retrieve a model corresponding at given ID and return it as a JSON
        object.
        ---
        tags:
          - Crud
        parameters:
          - in: path
            name: output_file_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            200:
                description: Model as a JSON object
            400:
                description: Statement error
            404:
                description: Value error
        """
        try:
            output_file = files_service.get_output_file(instance_id)
            self.check_read_permissions(output_file)
            return self.clean_get_result(output_file)

        except StatementError as exception:
            current_app.logger.error(str(exception), exc_info=1)
            return {"message": str(exception)}, 400

        except ValueError:
            abort(404)

    def post_update(self, instance_dict, data):
        files_service.clear_output_file_cache(instance_dict["id"])
        return instance_dict

    def post_delete(self, instance_dict):
        files_service.clear_output_file_cache(instance_dict["id"])
        return instance_dict

    def emit_update_event(self, instance_dict):
        entity = entities_service.get_entity(instance_dict["entity_id"])
        return events.emit(
            "output-file:update",
            {"output_file_id": instance_dict["id"]},
            project_id=entity["project_id"],
        )

    def emit_delete_event(self, instance_dict):
        entity = entities_service.get_entity(instance_dict["entity_id"])
        return events.emit(
            "output-file:delete",
            {"output_file_id": instance_dict["id"]},
            project_id=entity["project_id"],
        )



from sqlalchemy_utils import UUIDType

from zou.app import db
from zou.app.models.serializer import SerializerMixin
from zou.app.models.base import BaseMixin
from zou.app.utils import fields


class Milestone(db.Model, BaseMixin, SerializerMixin):
    """
    Allow to set a milestone date to the production schedule.
    """

    date = db.Column(db.Date())
    name = db.Column(db.String(40), nullable=False)

    project_id = db.Column(
        UUIDType(binary=False), db.ForeignKey("project.id"), index=True
    )
    task_type_id = db.Column(
        UUIDType(binary=False), db.ForeignKey("task_type.id"), index=True
    )

    def present(self):
        return fields.serialize_dict(
            {
                "id": self.id,
                "date": self.date,
                "name": self.name,
                "project_id": self.project_id,
                "task_type_id": self.task_type_id,
            }
        )



#!/usr/bin/env python

import warnings

warnings.filterwarnings("ignore")
import os
import sys
import flask_migrate
import click
import traceback

from sqlalchemy.exc import IntegrityError

from zou.app.utils import dbhelpers, auth, commands, plugins as plugin_utils
from zou.app.services import persons_service, auth_service, plugins_service
from zou.app.services.exception import (
    IsUserLimitReachedException,
    PersonNotFoundException,
    TwoFactorAuthenticationNotEnabledException,
)

from zou.app import app, config

from zou import __file__ as root_path

migrations_path = os.path.join(os.path.dirname(root_path), "migrations")


@click.group()
def cli():
    pass


@cli.command()
def version():
    """
    Return current installation version.
    """
    from zou import __version__

    print(f"Zou version: {__version__}")


@cli.command()
def init_db():
    "Create database table (database must be created through PG client)."

    print("Creating database and tables...")
    with app.app_context():
        flask_migrate.upgrade(directory=migrations_path)
    print("Database and tables created.")


@cli.command()
def is_db_ready():
    """
    Return a message telling whether the database is initialized or not.
    """
    with app.app_context():
        is_init = dbhelpers.is_init()
        if is_init:
            print("Database is initialized.")
        else:
            print(
                "Database is not initialized. "
                "Run 'zou init-db' and 'zou init-data'."
            )


@cli.command()
@click.option("--message", default="")
def migrate_db(message):
    """
    Generate migration files to describe a new revision of the database schema
    (for development only).
    """
    with app.app_context():
        flask_migrate.migrate(directory=migrations_path, message=message)


@cli.command()
@click.option("--revision", default="-1")
def downgrade_db(revision):
    """
    Downgrade db to previous revision of the database schema
    (for development only). For revision you can use an hash or a relative migration identifier.
    """
    with app.app_context():
        flask_migrate.downgrade(directory=migrations_path, revision=revision)


@cli.command()
def clear_db():
    "Drop all tables from database"

    with app.app_context():
        print("Deleting database and tables...")
        dbhelpers.drop_all()
        print("Database and tables deleted.")
        flask_migrate.stamp(directory=migrations_path, revision="base")


@cli.command()
def reset_db():
    "Drop all tables, then recreate them."
    with app.app_context():
        print("Deleting database and tables...")
        dbhelpers.drop_all()
        print("Database and tables deleted.")
        flask_migrate.stamp(directory=migrations_path, revision="base")
        flask_migrate.upgrade(directory=migrations_path)
        print("Database and tables created.")


@cli.command()
@click.option("--no-telemetry", is_flag=True, default=False)
def upgrade_db(no_telemetry=False):
    """
    Upgrade database schema. Send anonymized statistics to our telemetry
    services (user and preview amounts). It allows us to size the Kitsu
    community.
    """
    with app.app_context():
        flask_migrate.upgrade(directory=migrations_path)
        if not no_telemetry and config.IS_SELF_HOSTED:
            from zou.app.services import telemetry_services

            try:
                telemetry_services.send_main_infos()
            except Exception:
                traceback.print_exc()


@cli.command()
@click.option("--revision", default=None)
def stamp_db(revision):
    "Set the database schema revision to current one."
    with app.app_context():
        if revision is None:
            flask_migrate.stamp(directory=migrations_path)
        else:
            flask_migrate.stamp(directory=migrations_path, revision=revision)


@cli.command()
def clear_memory_cache():
    "Clear Redis memory cache."
    from zou.app import cache

    with app.app_context():
        cache.clear()


@cli.command()
def reset_migrations():
    "Set the database schema revision to first one."
    with app.app_context():
        flask_migrate.stamp(directory=migrations_path, revision="base")


@cli.command()
@click.argument("email")
@click.option("--password", default=None)
def create_admin(email, password):
    """
    Create an admin user to allow usage of the API when database is empty.
    """
    with app.app_context():
        try:
            person = persons_service.get_person_by_email(email)
            if person["role"] != "admin":
                persons_service.update_person(
                    person["id"],
                    {"role": "admin"},
                    bypass_protected_accounts=True,
                )
                print("Existing user's role has been upgraded to 'admin'.")
        except PersonNotFoundException:
            try:
                if password is None:
                    raise click.MissingParameter(
                        param_type="option", param_hint="--password"
                    )
                auth.validate_password(password)
                # Allow "admin@example.com" to be invalid.
                if email != "admin@example.com":
                    auth.validate_email(email)
                if persons_service.is_user_limit_reached():
                    raise IsUserLimitReachedException
                password = auth.encrypt_password(password)
                persons_service.create_person(
                    email, password, "Super", "Admin", role="admin"
                )
                print("Admin successfully created.")
            except IntegrityError:
                print("User already exists for this email.")
                sys.exit(1)
            except auth.PasswordTooShortException:
                print("Password is too short.")
                sys.exit(1)
            except auth.EmailNotValidException as e:
                print(f"Email is not valid: {e}")
                sys.exit(1)
            except IsUserLimitReachedException:
                print(f"User limit reached (limit {config.USER_LIMIT}).")
                sys.exit(1)


@cli.command()
def clean_auth_tokens():
    "Remove revoked and expired tokens."
    commands.clean_auth_tokens()


@cli.command()
def clear_all_auth_tokens():
    "Remove all authentication tokens."
    commands.clear_all_auth_tokens()


@cli.command()
def init_data():
    "Generate minimal data set required to run Kitsu."
    commands.init_data()


@cli.command()
@click.argument("email-or-desktop-login")
def disable_two_factor_authentication(email_or_desktop_login):
    """
    Disable two factor authentication for given user.
    """
    with app.app_context():
        try:
            person_id = persons_service.get_person_by_email_desktop_login(
                email_or_desktop_login
            )
            auth_service.disable_two_factor_authentication_for_person(
                person_id
            )
            print(
                f"Two factor authentication disabled for {email_or_desktop_login}."
            )
        except PersonNotFoundException:
            print(f"Email ({email_or_desktop_login}) not listed in database.")
            sys.exit(1)
        except TwoFactorAuthenticationNotEnabledException:
            print(
                f"Two factor authentication can't be disabled for {email_or_desktop_login} because it's not activated."
            )
            sys.exit(1)


@cli.command()
@click.argument("email")
@click.option("--password", required=True)
def change_password(email, password):
    """
    Change the password of given user.
    """
    with app.app_context():
        try:
            auth.validate_password(password)
            password = auth.encrypt_password(password)
            persons_service.update_password(email, password)
            print("Password changed for %s" % email)
        except auth.PasswordTooShortException:
            print("The password is too short.")
            sys.exit(1)


@cli.command()
@click.argument("email")
@click.option("--unactive", is_flag=True, default=False)
def set_person_as_active(email, unactive):
    """
    Set a person as active.
    """
    with app.app_context():
        try:
            if persons_service.is_user_limit_reached() and not unactive:
                raise IsUserLimitReachedException
            person = persons_service.get_person_by_email_raw(email)
            person.update({"active": not unactive})
            print(
                f'Person {email} is set as an {"active" if not unactive else "unactive"} user.'
            )
        except IsUserLimitReachedException:
            print(f"User limit reached (limit {config.USER_LIMIT}).")
            sys.exit(1)
        except PersonNotFoundException:
            print(f"Email ({email}) not listed in database.")
            sys.exit(1)


@cli.command()
def sync_with_ldap_server():
    """
    For each user account in your LDAP server, it creates a new user.
    """
    with app.app_context():
        if persons_service.is_user_limit_reached():
            print(
                "User limit reached (limit %i). New users will not be added."
                % config.USER_LIMIT
            )
            sys.exit(1)
        commands.sync_with_ldap_server()


@cli.command()
@click.option("--source", default="http://localhost:5000")
@click.option("--project")
@click.option("--no-projects", is_flag=True)
@click.option("--with-events", is_flag=True)
@click.option("--only-projects", is_flag=True)
def sync_full(
    source,
    project=None,
    with_events=False,
    no_projects=False,
    only_projects=False,
):
    """
    Retrieve all data from source instance. It expects that credentials to
    connect to source instance are given through SYNC_LOGIN and SYNC_PASSWORD
    environment variables.
    """
    print("Start syncing.")
    login = os.getenv("SYNC_LOGIN")
    password = os.getenv("SYNC_PASSWORD")
    commands.import_data_from_another_instance(
        source,
        login,
        password,
        project=project,
        with_events=with_events,
        no_projects=no_projects,
        only_projects=only_projects,
    )
    print("Syncing ended.")


@cli.command()
@click.option("--source", default="http://localhost:5000", show_default=True)
@click.option("--project", default=None, show_default=True)
@click.option(
    "--multithreaded", is_flag=True, show_default=True, default=False
)
@click.option("--number-workers", default=30, show_default=True, type=int)
@click.option("--number-attemps", default=3, show_default=True, type=int)
@click.option("--force-resync", is_flag=True, show_default=True, default=False)
def sync_full_files(
    source,
    project,
    multithreaded,
    number_workers,
    number_attemps,
    force_resync,
):
    """
    Retrieve all files from source instance. It expects that credentials to
    connect to source instance are given through SYNC_LOGIN and SYNC_PASSWORD
    environment variables.
    """
    print("Start syncing.")
    login = os.getenv("SYNC_LOGIN")
    password = os.getenv("SYNC_PASSWORD")
    dict_errors = commands.import_files_from_another_instance(
        source,
        login,
        password,
        project=project,
        multithreaded=multithreaded,
        number_workers=number_workers,
        number_attemps=number_attemps,
        force_resync=force_resync,
    )
    print("Syncing ended.")
    if dict_errors:
        print("Errors summary:")
        for prefix, value in dict_errors.items():
            print(f"{prefix}:")
            for id, error in value.items():
                print(f"{id}:\n{error}")
            print()
        sys.exit(1)


@cli.command()
@click.option("--event-source", default="http://localhost:8080")
@click.option("--source", default="http://localhost:8080/api")
@click.option("--logs-directory", default=None)
def sync_changes(event_source, source, logs_directory):
    """
    Run a daemon that import data related to any change happening on source
    instance. It expects that credentials to connect to source instance are
    given through SYNC_LOGIN and SYNC_PASSWORD environment variables.
    """
    login = os.getenv("SYNC_LOGIN")
    password = os.getenv("SYNC_PASSWORD")
    commands.run_sync_change_daemon(
        event_source, source, login, password, logs_directory
    )


@cli.command()
@click.option("--event-source", default="http://localhost:8080")
@click.option("--source", default="http://localhost:8080/api")
@click.option("--logs-directory", default=None)
def sync_file_changes(event_source, source, logs_directory):
    """
    Run a daemon that download files related to any change happening on source
    instance. It expects that credentials to connect to source instance are
    given through SYNC_LOGIN and SYNC_PASSWORD environment variables.
    """
    login = os.getenv("SYNC_LOGIN")
    password = os.getenv("SYNC_PASSWORD")
    commands.run_sync_file_change_daemon(
        event_source, source, login, password, logs_directory
    )


@cli.command()
@click.option("--source", default="http://localhost:8080/api")
@click.option("--minutes", default=0)
@click.option("--page-size", default=300)
def sync_last_events(source, minutes, limit):
    """
    Retrieve last events that occured on source instance and import data related
    to them. It expects that credentials to connect to source instance are
    given through SYNC_LOGIN and SYNC_PASSWORD environment variables.
    """
    login = os.getenv("SYNC_LOGIN")
    password = os.getenv("SYNC_PASSWORD")
    commands.import_last_changes_from_another_instance(
        source, login, password, minutes=minutes, limit=limit
    )


@cli.command()
@click.option("--source", default="http://localhost:8080/api")
@click.option("--minutes", default=20)
@click.option("--page-size", default=50)
def sync_last_files(source, minutes, limit):
    """
    Retrieve last preview files and thumbnails uploaded on source instance.
    It expects that credentials to connect to source instance are
    given through SYNC_LOGIN and SYNC_PASSWORD environment variables.
    """
    login = os.getenv("SYNC_LOGIN")
    password = os.getenv("SYNC_PASSWORD")
    commands.import_last_file_changes_from_another_instance(
        source, login, password, minutes=minutes, limit=limit
    )


@cli.command()
def download_storage_files():
    """
    Download all files from a Swift object storage and store them in a local
    storage.
    """
    commands.download_file_from_storage()


@cli.command()
@click.option("--store", is_flag=True)
def dump_database(store=False):
    """
    Dump database described in Zou environment variables and save it to
    configured object storage.
    """
    commands.dump_database(store)


@cli.command()
@click.option("--days", default=None)
def upload_files_to_cloud_storage(days):
    """
    Upload all files related to previews to configured object storage.
    """
    commands.upload_files_to_cloud_storage(days)


@cli.command()
@click.option("--project-id")
def clean_tasks_data(project_id):
    """
    Reset task models data (retake count, wip start date and end date)
    """
    if project_id is not None:
        commands.reset_tasks_data(project_id)


@cli.command()
@click.option("--days", default=90)
def remove_old_data(days):
    """
    Remove old events, notifications and login logs older than 90 days
    (by default).
    """
    commands.remove_old_data(days)


@cli.command()
def reset_search_index():
    """
    Reset search index.
    """
    commands.reset_search_index()


@cli.command()
@click.option("--query", default="")
def search_asset(query):
    """ """
    commands.search_asset(query)


@cli.command()
@click.option("--project", default=None, show_default=True)
@click.option("--entity-id", default=None, show_default=True)
@click.option("--episode", default=None, show_default=True, multiple=True)
@click.option("--only-shots", is_flag=True, default=False, show_default=True)
@click.option("--only-assets", is_flag=True, default=False, show_default=True)
@click.option("--with-tiles", is_flag=True, default=False, show_default=True)
@click.option(
    "--with-metadata", is_flag=True, default=False, show_default=True
)
@click.option(
    "--with-thumbnails", is_flag=True, default=False, show_default=True
)
@click.option(
    "--force-regenerate-tiles", is_flag=True, default=False, show_default=True
)
def generate_preview_extra(
    project,
    entity_id,
    episode,
    only_shots,
    only_assets,
    with_tiles,
    with_metadata,
    with_thumbnails,
    force_regenerate_tiles,
):
    """
    Generate tiles, thumbnails and metadata for all previews.
    """
    commands.generate_preview_extra(
        project=project,
        entity_id=entity_id,
        episodes=episode,
        only_shots=only_shots,
        only_assets=only_assets,
        force_regenerate_tiles=force_regenerate_tiles,
        with_tiles=with_tiles,
        with_metadata=with_metadata,
        with_thumbnails=with_thumbnails,
    )


@cli.command()
def reset_movie_files_metadata():
    """
    Store height and width metadata for all movie previews in the database.
    """
    commands.reset_movie_files_metadata()


@cli.command()
def reset_picture_files_metadata():
    """
    Store height and width metadata for all picture previews in the database.
    """
    commands.reset_picture_files_metadata()


@cli.command()
def reset_breakdown_data():
    """
    Reset breakdown statistics for all open projects.
    """
    commands.reset_breakdown_data()


@cli.command()
@click.option("--email", required=True)
@click.option("--name", required=True)
@click.option(
    "--expiration-date",
    required=False,
    default=None,
    show_default=True,
    help="Format: YYYY-MM-DD",
)
@click.option("--role", required=False, default="user", show_default=True)
def create_bot(
    email,
    name,
    expiration_date,
    role,
):
    """
    Create a bot.
    """
    commands.create_bot(
        email,
        name,
        expiration_date,
        role,
    )


@cli.command()
@click.option(
    "--preview-file-id",
    required=False,
    default=None,
    show_default=True,
)
@click.option(
    "--project-id",
    required=False,
    default=None,
    show_default=True,
)
@click.option("--all-broken", is_flag=True, default=False, show_default=True)
@click.option(
    "--all-processing", is_flag=True, default=False, show_default=True
)
@click.option("--days", type=int, default=None, show_default=True)
@click.option("--hours", type=int, default=None, show_default=True)
@click.option("--minutes", type=int, default=None, show_default=True)
def renormalize_movie_preview_files(
    preview_file_id,
    project_id,
    all_broken,
    all_processing,
    days=None,
    hours=None,
    minutes=None,
):
    """
    Renormalize all preview files.
    """
    commands.renormalize_movie_preview_files(
        preview_file_id,
        project_id,
        all_broken,
        all_processing,
        days=days,
        hours=hours,
        minutes=minutes,
    )


@cli.command()
@click.option(
    "--path",
    required=True,
)
@click.option(
    "--force",
    is_flag=True,
    default=False,
    show_default=True,
)
def install_plugin(path, force=False):
    """
    Install a plugin.
    """
    with app.app_context():
        plugins_service.install_plugin(path, force)
    print(f"Plugin {path} installed. Restart the server to apply changes.")


@cli.command()
@click.option(
    "--id",
    required=True,
)
def uninstall_plugin(id):
    """
    Uninstall a plugin.
    """
    with app.app_context():
        plugins_service.uninstall_plugin(id)
    print(f"Plugin {id} uninstalled.")


@cli.command()
@click.option(
    "--path",
    required=True,
)
@click.option(
    "--id",
    help="Plugin ID (must be unique).",
    required=True,
)
@click.option(
    "--name", help="Plugin name.", default="MyPlugin", show_default=True
)
@click.option(
    "--description",
    help="Plugin description.",
    default="My plugin description.",
    show_default=True,
)
@click.option(
    "--version", help="Plugin version.", default="0.1.0", show_default=True
)
@click.option(
    "--maintainer",
    help="Plugin maintainer.",
    default="Author <author@author.com>",
    show_default=True,
)
@click.option(
    "--website",
    help="Plugin website.",
    default="mywebsite.com",
    show_default=True,
)
@click.option(
    "--license",
    help="Plugin license.",
    default="GPL-3.0-only",
    show_default=True,
)
@click.option(
    "--force",
    is_flag=True,
    default=False,
    show_default=True,
)
def create_plugin_skeleton(
    path,
    id,
    name,
    description,
    version,
    maintainer,
    website,
    license,
    force=False,
):
    """
    Create a plugin skeleton.
    """
    plugin_path = plugin_utils.create_plugin_skeleton(
        path,
        id,
        name,
        description,
        version,
        maintainer,
        website,
        license,
        force,
    )
    print(f"Plugin skeleton created in '{plugin_path}'.")


@cli.command()
@click.option(
    "--path",
    required=True,
)
@click.option(
    "--output-path",
    required=True,
)
@click.option(
    "--force",
    is_flag=True,
    default=False,
    show_default=True,
)
def create_plugin_package(
    path,
    output_path,
    force=False,
):
    """
    Create a plugin package.
    """
    plugin_path = plugin_utils.create_plugin_package(path, output_path, force)
    print(f"Plugin package created in '{plugin_path}'.")


@cli.command()
@click.option(
    "--format",
    "output_format",
    type=click.Choice(["table", "json"], case_sensitive=False),
    default="table",
    show_default=True,
    help="Output format: table or json.",
)
@click.option(
    "--verbose",
    is_flag=True,
    default=False,
    help="Show more plugin information.",
)
@click.option(
    "--filter-field",
    type=click.Choice(
        ["plugin_id", "name", "maintainer", "license"], case_sensitive=False
    ),
    help="Field to filter by.",
)
@click.option("--filter-value", type=str, help="Value to search in the field.")
def list_plugins(output_format, verbose, filter_field, filter_value):
    """
    List installed plugins.
    """
    commands.list_plugins(output_format, verbose, filter_field, filter_value)


@cli.command()
@click.option(
    "--path",
    required=True,
)
@click.option("--message", default="")
def migrate_plugin_db(path, message):
    """
    Migrate plugin database.
    """
    with app.app_context():
        plugin_utils.migrate_plugin_db(path, message)
    print(f"Plugin {path} database migrated.")


if __name__ == "__main__":
    cli()



from zou.app.models.subscription import Subscription

from zou.app.blueprints.crud.base import BaseModelResource, BaseModelsResource


class SubscriptionsResource(BaseModelsResource):
    def __init__(self):
        BaseModelsResource.__init__(self, Subscription)


class SubscriptionResource(BaseModelResource):
    def __init__(self):
        BaseModelResource.__init__(self, Subscription)
